# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import re

from PySide6.QtGui import QValidator
from PySide6.QtCore import QByteArray, Signal, Slot
from PySide6.QtWidgets import QGroupBox
from PySide6.QtSerialBus import QCanBusFrame

from ui_sendframebox import Ui_SendFrameBox


THREE_HEX_DIGITS_PATTERN = re.compile("[0-9a-fA-F]{3}")
HEX_NUMBER_PATTERN = re.compile("^[0-9a-fA-F]+$")


MAX_STANDARD_ID = 0x7FF
MAX_EXTENDED_ID = 0x10000000
MAX_PAYLOAD = 8
MAX_PAYLOAD_FD = 64


def is_even_hex(input):
    return len(input.replace(" ", "")) % 2 == 0


def insert_space(string, pos):
    return string[0:pos] + " " + string[pos:]


# Formats a string of hex characters with a space between every byte
# Example: "012345" -> "01 23 45"
def format_hex_data(input):
    out = input.strip()
    while True:
        match = THREE_HEX_DIGITS_PATTERN.search(out)
        if match:
            out = insert_space(out, match.end(0) - 1)
        else:
            break
    return out.strip().upper()


class HexIntegerValidator(QValidator):

    def __init__(self, parent):
        super().__init__(parent)
        self.m_maximum = MAX_STANDARD_ID

    def validate(self, input, pos):
        result = QValidator.Intermediate
        if input:
            result = QValidator.Invalid
            try:
                value = int(input, base=16)
                if value < self.m_maximum:
                    result = QValidator.Acceptable
            except ValueError:
                pass
        return result

    def set_maximum(self, maximum):
        self.m_maximum = maximum


class HexStringValidator(QValidator):

    def __init__(self, parent):
        super().__init__(parent)
        self.m_maxLength = MAX_PAYLOAD

    def validate(self, input, pos):
        max_size = 2 * self.m_maxLength
        data = input.replace(" ", "")
        if not data:
            return QValidator.Intermediate

        # limit maximum size
        if len(data) > max_size:
            return QValidator.Invalid

        # check if all input is valid
        if not HEX_NUMBER_PATTERN.match(data):
            return QValidator.Invalid

        # insert a space after every two hex nibbles
        while True:
            match = THREE_HEX_DIGITS_PATTERN.search(input)
            if not match:
                break
            start = match.start(0)
            end = match.end()
            if pos == start + 1:
                # add one hex nibble before two - Abc
                input = insert_space(input, pos)
            elif pos == start + 2:
                # add hex nibble in the middle - aBc
                input = insert_space(input, end - 1)
                pos = end
            else:
                # add one hex nibble after two - abC
                input = insert_space(input, end - 1)
                pos = end + 1

        return (QValidator.Acceptable, input, pos)

    def set_max_length(self, maxLength):
        self.m_maxLength = maxLength


class SendFrameBox(QGroupBox):

    send_frame = Signal(QCanBusFrame)

    def __init__(self, parent):
        super().__init__(parent)
        self.m_ui = Ui_SendFrameBox()
        self.m_ui.setupUi(self)

        self.m_hexIntegerValidator = HexIntegerValidator(self)
        self.m_ui.frameIdEdit.setValidator(self.m_hexIntegerValidator)
        self.m_hexStringValidator = HexStringValidator(self)
        self.m_ui.payloadEdit.setValidator(self.m_hexStringValidator)

        self.m_ui.dataFrame.toggled.connect(self._data_frame)
        self.m_ui.remoteFrame.toggled.connect(self._remote_frame)
        self.m_ui.errorFrame.toggled.connect(self._error_frame)
        self.m_ui.extendedFormatBox.toggled.connect(self._extended_format)
        self.m_ui.flexibleDataRateBox.toggled.connect(self._flexible_datarate)
        self.m_ui.frameIdEdit.textChanged.connect(self._frameid_or_payload_changed)
        self.m_ui.payloadEdit.textChanged.connect(self._frameid_or_payload_changed)
        self._frameid_or_payload_changed()
        self.m_ui.sendButton.clicked.connect(self._send)

    @Slot(bool)
    def _data_frame(self, value):
        if value:
            self.m_ui.flexibleDataRateBox.setEnabled(True)

    @Slot(bool)
    def _remote_frame(self, value):
        if value:
            self.m_ui.flexibleDataRateBox.setEnabled(False)
            self.m_ui.flexibleDataRateBox.setChecked(False)

    @Slot(bool)
    def _error_frame(self, value):
        if value:
            self.m_ui.flexibleDataRateBox.setEnabled(False)
            self.m_ui.flexibleDataRateBox.setChecked(False)

    @Slot(bool)
    def _extended_format(self, value):
        m = MAX_EXTENDED_ID if value else MAX_STANDARD_ID
        self.m_hexIntegerValidator.set_maximum(m)

    @Slot(bool)
    def _flexible_datarate(self, value):
        len = MAX_PAYLOAD_FD if value else MAX_PAYLOAD
        self.m_hexStringValidator.set_max_length(len)
        self.m_ui.bitrateSwitchBox.setEnabled(value)
        if not value:
            self.m_ui.bitrateSwitchBox.setChecked(False)

    @Slot()
    def _frameid_or_payload_changed(self):
        has_frame_id = bool(self.m_ui.frameIdEdit.text())
        self.m_ui.sendButton.setEnabled(has_frame_id)
        tt = "" if has_frame_id else "Cannot send because no Frame ID was given."
        self.m_ui.sendButton.setToolTip(tt)
        if has_frame_id:
            is_even = is_even_hex(self.m_ui.payloadEdit.text())
            self.m_ui.sendButton.setEnabled(is_even)
            tt = "" if is_even else "Cannot send because Payload hex string is invalid."
            self.m_ui.sendButton.setToolTip(tt)

    @Slot()
    def _send(self):
        frame_id = int(self.m_ui.frameIdEdit.text(), base=16)
        data = self.m_ui.payloadEdit.text().replace(" ", "")
        self.m_ui.payloadEdit.setText(format_hex_data(data))
        payload = QByteArray.fromHex(bytes(data, encoding='utf8'))

        frame = QCanBusFrame(frame_id, payload)
        frame.setExtendedFrameFormat(self.m_ui.extendedFormatBox.isChecked())
        frame.setFlexibleDataRateFormat(self.m_ui.flexibleDataRateBox.isChecked())
        frame.setBitrateSwitch(self.m_ui.bitrateSwitchBox.isChecked())

        if self.m_ui.errorFrame.isChecked():
            frame.setFrameType(QCanBusFrame.ErrorFrame)
        elif self.m_ui.remoteFrame.isChecked():
            frame.setFrameType(QCanBusFrame.RemoteRequestFrame)

        self.send_frame.emit(frame)



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Qt RESTful API client demo from Qt v6.x"""

import os
import sys
from pathlib import Path

from PySide6.QtCore import QUrl
from PySide6.QtGui import QIcon, QGuiApplication
from PySide6.QtQml import QQmlApplicationEngine

from basiclogin import BasicLogin  # noqa: F401
from paginatedresource import PaginatedResource  # noqa: F401
from restservice import RestService  # noqa: F401
import rc_colorpaletteclient  # noqa: F401

if __name__ == "__main__":
    app = QGuiApplication(sys.argv)
    QIcon.setThemeName("colorpaletteclient")

    engine = QQmlApplicationEngine()
    app_dir = Path(__file__).parent
    app_dir_url = QUrl.fromLocalFile(os.fspath(app_dir))
    engine.addImportPath(os.fspath(app_dir))
    engine.loadFromModule("ColorPalette", "Main")
    if not engine.rootObjects():
        sys.exit(-1)

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWebEngineCore import QWebEngineDownloadRequest
from PySide6.QtWidgets import QWidget, QFileDialog
from PySide6.QtCore import QDir, QFileInfo, Qt

from downloadwidget import DownloadWidget
from ui_downloadmanagerwidget import Ui_DownloadManagerWidget


# Displays a list of downloads.
class DownloadManagerWidget(QWidget):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._ui = Ui_DownloadManagerWidget()
        self._num_downloads = 0
        self._ui.setupUi(self)

    def download_requested(self, download):
        assert (download and download.state() == QWebEngineDownloadRequest.DownloadRequested)

        proposal_dir = download.downloadDirectory()
        proposal_name = download.downloadFileName()
        proposal = QDir(proposal_dir).filePath(proposal_name)
        path, _ = QFileDialog.getSaveFileName(self, "Save as", proposal)
        if not path:
            return

        fi = QFileInfo(path)
        download.setDownloadDirectory(fi.path())
        download.setDownloadFileName(fi.fileName())
        download.accept()
        self.add(DownloadWidget(download))

        self.show()

    def add(self, downloadWidget):
        downloadWidget.remove_clicked.connect(self.remove)
        self._ui.m_itemsLayout.insertWidget(0, downloadWidget, 0, Qt.AlignTop)
        if self._num_downloads == 0:
            self._ui.m_zeroItemsLabel.hide()
        self._num_downloads += 1

    def remove(self, downloadWidget):
        self._ui.m_itemsLayout.removeWidget(downloadWidget)
        downloadWidget.deleteLater()
        self._num_downloads -= 1
        if self._num_downloads == 0:
            self._ui.m_zeroItemsLabel.show()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QImage/Windows'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt
from PySide6.QtGui import QImage
from helper.usesqapplication import UsesQApplication


def create_image():
    result = QImage(20, 20, QImage.Format.Format_RGB32)
    result.fill(Qt.GlobalColor.white)
    return result


class QImageWinTest(UsesQApplication):

    def test_to_hbitmap(self):
        """Test conversion to/from a Windows HBITMAP."""

        image = create_image()
        hbitmap = image.toHBITMAP()
        self.assertTrue(hbitmap > 0)

        image2 = QImage.fromHBITMAP(hbitmap)
        image2.setColorSpace(image.colorSpace())
        self.assertEqual(image, image2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Slot, Signal


class Sender(QObject):
    mySignal = Signal()


class MyObject(QObject):
    def __init__(self, parent=None):
        QObject.__init__(self, parent)
        self._slotCalledCount = 0

    @Slot()
    def mySlot(self):
        self._slotCalledCount = self._slotCalledCount + 1

    @Slot(int)
    @Slot('QString')
    def mySlot2(self, arg0):
        self._slotCalledCount = self._slotCalledCount + 1

    @Slot(name='mySlot3')
    def foo(self):
        self._slotCalledCount = self._slotCalledCount + 1

    @Slot(str, int)
    def mySlot4(self, a, b):
        self._slotCalledCount = self._slotCalledCount + 1

    @Slot(result=int)
    def mySlot5(self):
        self._slotCalledCount = self._slotCalledCount + 1

    @Slot(result=QObject)
    def mySlot6(self):
        self._slotCalledCount = self._slotCalledCount + 1


class StaticMetaObjectTest(unittest.TestCase):

    def testSignalPropagation(self):
        o = MyObject()
        m = o.metaObject()
        self.assertTrue(m.indexOfSlot('mySlot()') > 0)
        self.assertTrue(m.indexOfSlot('mySlot2(int)') > 0)
        self.assertTrue(m.indexOfSlot('mySlot2(QString)') > 0)
        self.assertTrue(m.indexOfSlot('mySlot3()') > 0)
        self.assertTrue(m.indexOfSlot('mySlot4(QString,int)') > 0)

    def testEmission(self):
        sender = Sender()
        o = MyObject()
        sender.mySignal.connect(o.mySlot)
        sender.mySignal.emit()
        self.assertTrue(o._slotCalledCount == 1)

    def testResult(self):
        o = MyObject()
        mo = o.metaObject()
        i = mo.indexOfSlot('mySlot5()')
        m = mo.method(i)
        self.assertEqual(m.typeName(), "int")

    def testResultObject(self):
        o = MyObject()
        mo = o.metaObject()
        i = mo.indexOfSlot('mySlot6()')
        m = mo.method(i)
        self.assertEqual(m.typeName(), "QObject*")


class SlotWithoutArgs(unittest.TestCase):

    def testError(self):
        # It should be an error to call the slot without the
        # arguments, as just @Slot would end up in a slot
        # accepting argument functions
        self.assertRaises(TypeError, Slot, lambda: 3)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QTimer, Property, Signal, Slot
from PySide6.QtQml import QmlElement, QPyQmlPropertyValueSource

# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "People"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class HappyBirthdaySong(QPyQmlPropertyValueSource):
    name_changed = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)

        self.m_target = None
        self.m_name = ""
        self.m_line = -1
        self.m_lyrics = []

        self.m_timer = QTimer(self)
        self.m_timer.timeout.connect(self.advance)
        self.m_timer.start(1000)

    def setTarget(self, property):
        self.m_target = property

    @Property(str, notify=name_changed, final=True)
    def name(self):
        return self.m_name

    @name.setter
    def name(self, n):
        if self.m_name != n:
            self.m_name = n
            self.m_lyrics = ["Happy birthday to you,",
                             "Happy birthday to you,",
                             f"Happy birthday dear {self.m_name},",
                             "Happy birthday to you!",
                             ""]

    @Slot()
    def advance(self):
        self.m_line = (self.m_line + 1) % len(self.m_lyrics)
        self.m_target.write(self.m_lyrics[self.m_line])



# Copyright (C) 2023 The Qt Company Ltd.
# Copyright (C) 2013 Riverbank Computing Limited.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import math

from PySide6.QtGui import (QVector3D, QVector3DList)


class Logo():
    def __init__(self):
        self.m_data = QVector3DList()
        self.m_data.reserve(5000)

        x1 = +0.06
        y1 = -0.14
        x2 = +0.14
        y2 = -0.06
        x3 = +0.08
        y3 = +0.00
        x4 = +0.30
        y4 = +0.22

        self.quad(x1, y1, x2, y2, y2, x2, y1, x1)
        self.quad(x3, y3, x4, y4, y4, x4, y3, x3)

        self.extrude(x1, y1, x2, y2)
        self.extrude(x2, y2, y2, x2)
        self.extrude(y2, x2, y1, x1)
        self.extrude(y1, x1, x1, y1)
        self.extrude(x3, y3, x4, y4)
        self.extrude(x4, y4, y4, x4)
        self.extrude(y4, x4, y3, x3)

        NUM_SECTORS = 100

        for i in range(NUM_SECTORS):
            angle = (i * 2 * math.pi) / NUM_SECTORS
            x5 = 0.30 * math.sin(angle)
            y5 = 0.30 * math.cos(angle)
            x6 = 0.20 * math.sin(angle)
            y6 = 0.20 * math.cos(angle)

            angle = ((i + 1) * 2 * math.pi) / NUM_SECTORS
            x7 = 0.20 * math.sin(angle)
            y7 = 0.20 * math.cos(angle)
            x8 = 0.30 * math.sin(angle)
            y8 = 0.30 * math.cos(angle)

            self.quad(x5, y5, x6, y6, x7, y7, x8, y8)

            self.extrude(x6, y6, x7, y7)
            self.extrude(x8, y8, x5, y5)

    def const_data(self):
        return self.m_data.constData()

    def count(self):
        return len(self.m_data) * 3

    def vertex_count(self):
        return self.count() / 6

    def quad(self, x1, y1, x2, y2, x3, y3, x4, y4):
        n = QVector3D.normal(QVector3D(x4 - x1, y4 - y1, 0),
                             QVector3D(x2 - x1, y2 - y1, 0))

        self.add(QVector3D(x1, y1, -0.05), n)
        self.add(QVector3D(x4, y4, -0.05), n)
        self.add(QVector3D(x2, y2, -0.05), n)

        self.add(QVector3D(x3, y3, -0.05), n)
        self.add(QVector3D(x2, y2, -0.05), n)
        self.add(QVector3D(x4, y4, -0.05), n)

        n = QVector3D.normal(QVector3D(x1 - x4, y1 - y4, 0),
                             QVector3D(x2 - x4, y2 - y4, 0))

        self.add(QVector3D(x4, y4, 0.05), n)
        self.add(QVector3D(x1, y1, 0.05), n)
        self.add(QVector3D(x2, y2, 0.05), n)

        self.add(QVector3D(x2, y2, 0.05), n)
        self.add(QVector3D(x3, y3, 0.05), n)
        self.add(QVector3D(x4, y4, 0.05), n)

    def extrude(self, x1, y1, x2, y2):
        n = QVector3D.normal(QVector3D(0, 0, -0.1),
                             QVector3D(x2 - x1, y2 - y1, 0))

        self.add(QVector3D(x1, y1, 0.05), n)
        self.add(QVector3D(x1, y1, -0.05), n)
        self.add(QVector3D(x2, y2, 0.05), n)

        self.add(QVector3D(x2, y2, -0.05), n)
        self.add(QVector3D(x2, y2, 0.05), n)
        self.add(QVector3D(x1, y1, -0.05), n)

    def add(self, v, n):
        self.m_data.append(v)
        self.m_data.append(n)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest
import weakref

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QMainWindow, QPushButton, QToolButton, QWidget
from helper.usesqapplication import UsesQApplication


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.createToolbar()

    def createToolbar(self):
        pointerButton = QToolButton()
        pointerToolbar = self.addToolBar("Pointer type")
        pointerToolbar.addWidget(pointerButton)


class MyButton(QPushButton):
    def __init__(self, parent=None):
        super().__init__()
        self._called = False

    def myCallback(self):
        self._called = True


class TestMainWindow(UsesQApplication):

    def objDel(self, obj):
        self.app.quit()

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountToNull(self):
        w = QMainWindow()
        c = QWidget()
        base_ref_count = sys.getrefcount(c)
        w.setCentralWidget(c)
        self.assertEqual(sys.getrefcount(c), base_ref_count + 1)
        wr = weakref.ref(c, self.objDel)  # noqa: F841
        w.setCentralWidget(None)
        c = None
        self.app.exec()

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountToAnother(self):
        w = QMainWindow()
        c = QWidget()
        base_ref_count = sys.getrefcount(c)
        w.setCentralWidget(c)
        self.assertEqual(sys.getrefcount(c), base_ref_count + 1)

        c2 = QWidget()
        base_ref_count = sys.getrefcount(c2)
        w.setCentralWidget(c2)
        self.assertEqual(sys.getrefcount(c2), base_ref_count + 1)

        wr = weakref.ref(c, self.objDel)  # noqa: F841
        w.setCentralWidget(None)
        c = None

        self.app.exec()

    def testSignalDisconect(self):
        w = QMainWindow()
        b = MyButton("button")
        b.clicked.connect(b.myCallback)
        w.setCentralWidget(b)

        b = MyButton("button")
        b.clicked.connect(b.myCallback)
        w.setCentralWidget(b)

        b.click()
        self.assertEqual(b._called, True)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import inspect
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

import PySide6.QtCore
import PySide6.QtWidgets
from PySide6.support.signature import get_signature


class PySideSignatureTest(unittest.TestCase):
    def testSignatureExist(self):
        t1 = type(get_signature(PySide6.QtCore.QObject.children))
        self.assertEqual(t1, inspect.Signature)
        t2 = type(get_signature(PySide6.QtCore.QObject.__dict__["children"]))
        self.assertEqual(t2, t1)
        obj = PySide6.QtWidgets.QApplication.palette
        t3 = type(get_signature(obj))
        self.assertEqual(t3, list)
        self.assertEqual(len(get_signature(obj)), 3)
        for thing in get_signature(obj):
            self.assertEqual(type(thing), inspect.Signature)
        sm = PySide6.QtWidgets.QApplication.__dict__["palette"]
        # PYSIDE-1436: staticmethod is a callable since Python 3.10
        # Instead of checking callable(sm), we check the type:
        self.assertEqual(type(sm), staticmethod)
        self.assertTrue(get_signature(sm) is not None)

    def testSignatureIsCached(self):
        # see if we get the same object
        ob1 = get_signature(PySide6.QtCore.QObject.children)
        ob2 = get_signature(PySide6.QtCore.QObject.children)
        self.assertTrue(ob1 is ob2)
        # same with multi signature
        ob1 = get_signature(PySide6.QtWidgets.QApplication.palette)
        ob2 = get_signature(PySide6.QtWidgets.QApplication.palette)
        self.assertTrue(ob1 is ob2)

    def testModuleIsInitialized(self):
        self.assertTrue(get_signature(PySide6.QtWidgets.QApplication) is not None)

    def test_NotCalled_is_callable_and_correct(self):
        # A signature that has a default value with some "Default(...)"
        # wrapper is callable and creates an object of the right type.
        sig = get_signature(PySide6.QtCore.QByteArray().toPercentEncoding)
        called_default = sig.parameters["exclude"].default()
        self.assertEqual(type(called_default), PySide6.QtCore.QByteArray)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

import struct

from PySide6.QtCore import Qt, Property, QUrl, Signal, QFile, QPointF
from PySide6.QtGui import QPen, QPainter
from PySide6.QtMultimedia import QAudioFormat, QAudioDecoder
from PySide6.QtQml import QmlElement
from PySide6.QtQuick import QQuickPaintedItem

QML_IMPORT_NAME = "Audio"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class WaveformItem(QQuickPaintedItem):

    fileChanged = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._waveformData = []
        self._background_color = Qt.black

        audio_format = QAudioFormat()
        audio_format.setChannelCount(1)
        audio_format.setSampleRate(44100)
        audio_format.setSampleFormat(QAudioFormat.Float)

        self._file_url: QUrl | None = None
        self._audio_file: QFile | None = None

        self._decoder = QAudioDecoder()
        self._decoder.setAudioFormat(audio_format)

        self._decoder.bufferReady.connect(self.onBufferReady)
        self._decoder.finished.connect(self.decoderFinished)

    def file(self) -> QUrl | None:
        return self._file_url

    def setFile(self, value: QUrl):
        if self._decoder.source() == value:
            return

        if self._audio_file and self._audio_file.isOpen():
            self._audio_file.close()

        self._waveformData = []
        self._decoder.stop()

        self._file_url = value
        if "__compiled__" in globals():
            path = self._file_url.toString().replace("qrc:/", ":/")
        else:
            path = self._file_url.path()
        self._audio_file = QFile(path)
        self._audio_file.open(QFile.ReadOnly)
        self._decoder.setSourceDevice(self._audio_file)
        self._decoder.start()
        self.fileChanged.emit()

    def paint(self, painter):
        # Fill the bounding rectangle with the specified color
        painter.fillRect(self.boundingRect(), self._background_color)

        # If no waveform data is available, draw the text
        if not self._waveformData:
            painter.setPen(Qt.white)
            painter.drawText(self.boundingRect(), Qt.AlignCenter, "Waveform not available")
            return

        painter.setRenderHint(QPainter.Antialiasing)

        # Set the pen for drawing the waveform
        pen = QPen(Qt.blue)
        pen.setWidth(1)
        painter.setPen(pen)

        # Get container dimensions
        rect = self.boundingRect()
        data_size = len(self._waveformData)

        # Calculate step size and center line
        x_step = rect.width() / data_size
        center_y = rect.height() / 2.0

        # Draw the waveform as connected lines
        for i in range(1, data_size):
            x1 = (i - 1) * x_step
            y1 = center_y - self._waveformData[i - 1] * center_y
            x2 = i * x_step
            y2 = center_y - self._waveformData[i] * center_y
            painter.drawLine(QPointF(x1, y1), QPointF(x2, y2))

    @staticmethod
    def float_buffer_to_list(data):
        # Calculate the number of 32-bit floats in the buffer
        float_count = len(data) // 4  # Each float32 is 4 bytes
        # Unpack the binary data into a list of floats
        return list(struct.unpack(f"{float_count}f", data))

    def onBufferReady(self):
        buffer = self._decoder.read()
        data = buffer.constData()
        self._waveformData.extend(self.float_buffer_to_list(data))
        self.update()

    file: QUrl = Property(QUrl, file, setFile, notify=fileChanged)

    def decoderFinished(self):
        self._audio_file.close()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import copy
from pathlib import Path

from PySide6.QtSql import QSqlRelationalDelegate
from PySide6.QtWidgets import QSpinBox, QStyle
from PySide6.QtGui import QPixmap, QPalette
from PySide6.QtCore import QEvent, QSize, Qt


class BookDelegate(QSqlRelationalDelegate):
    """Books delegate to rate the books"""

    def __init__(self, star_png, parent=None):
        QSqlRelationalDelegate.__init__(self, parent)
        star_png = Path(__file__).parent / "images" / "star.png"
        self.star = QPixmap(star_png)

    def paint(self, painter, option, index):
        """ Paint the items in the table.

            If the item referred to by <index> is a StarRating, we
            handle the painting ourselves. For the other items, we
            let the base class handle the painting as usual.

            In a polished application, we'd use a better check than
            the column number to find out if we needed to paint the
            stars, but it works for the purposes of this example.
        """
        if index.column() != 5:
            # Since we draw the grid ourselves:
            opt = copy.copy(option)
            opt.rect = option.rect.adjusted(0, 0, -1, -1)
            QSqlRelationalDelegate.paint(self, painter, opt, index)
        else:
            model = index.model()
            if option.state & QStyle.State_Enabled:
                if option.state & QStyle.State_Active:
                    color_group = QPalette.Normal
                else:
                    color_group = QPalette.Inactive
            else:
                color_group = QPalette.Disabled

            if option.state & QStyle.State_Selected:
                painter.fillRect(option.rect,
                                 option.palette.color(color_group, QPalette.Highlight))
            rating = model.data(index, Qt.DisplayRole)
            width = self.star.width()
            height = self.star.height()
            x = option.rect.x()
            y = option.rect.y() + (option.rect.height() / 2) - (height / 2)
            for i in range(rating):
                painter.drawPixmap(x, y, self.star)
                x += width

            # Since we draw the grid ourselves:
            self.drawFocus(painter, option, option.rect.adjusted(0, 0, -1, -1))

        pen = painter.pen()
        painter.setPen(option.palette.color(QPalette.Mid))
        painter.drawLine(option.rect.bottomLeft(), option.rect.bottomRight())
        painter.drawLine(option.rect.topRight(), option.rect.bottomRight())
        painter.setPen(pen)

    def sizeHint(self, option, index):
        """ Returns the size needed to display the item in a QSize object. """
        if index.column() == 5:
            size_hint = QSize(5 * self.star.width(), self.star.height()) + QSize(1, 1)
            return size_hint
        # Since we draw the grid ourselves:
        return QSqlRelationalDelegate.sizeHint(self, option, index) + QSize(1, 1)

    def editorEvent(self, event, model, option, index):
        if index.column() != 5:
            return False

        if event.type() == QEvent.MouseButtonPress:
            mouse_pos = event.pos()
            new_stars = int(0.7 + (mouse_pos.x() - option.rect.x()) / self.star.width())
            stars = max(0, min(new_stars, 5))
            model.setData(index, stars)
            # So that the selection can change
            return False

        return True

    def createEditor(self, parent, option, index):
        if index.column() != 4:
            return QSqlRelationalDelegate.createEditor(self, parent, option, index)

        # For editing the year, return a spinbox with a range from -1000 to 2100.
        spinbox = QSpinBox(parent)
        spinbox.setFrame(False)
        spinbox.setMaximum(2100)
        spinbox.setMinimum(-1000)
        return spinbox



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Property
from PySide6.QtQml import QmlElement, ListProperty
from PySide6.QtQuick import QQuickItem

from pieslice import PieSlice

# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "Charts"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class PieChart(QQuickItem):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._slices = []
        self._name = ''

    @Property(str, final=True)
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    def slice(self, n):
        return self._slices[n]

    def sliceCount(self):
        return len(self._slices)

    def append_and_setparent(self, slice):
        self._slices.append(slice)
        slice.setParentItem(self)

    slices = ListProperty(PieSlice, append_and_setparent)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal

MAX_LOOPS = 5
MAX_OBJECTS = 200


class Sender(QObject):
    fire = Signal()


class MultipleSlots(unittest.TestCase):
    def myCB(self):
        self._count += 1

    def testDisconnectCleanup(self):
        for c in range(MAX_LOOPS):
            self._count = 0
            self._senders = []
            for i in range(MAX_OBJECTS):
                o = Sender()
                o.fire.connect(lambda: self.myCB())
                self._senders.append(o)
                o.fire.emit()

            self.assertEqual(self._count, MAX_OBJECTS)

            # delete all senders will disconnect the signals
            self._senders = []


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
sys.path.append(os.fspath(Path(__file__).resolve().parents[1] / "util"))
from init_paths import init_test_paths
init_test_paths()

from PySide6.QtCore import QCoreApplication, QObject, QTimer


class Dispatcher(QObject):
    _me = None

    def __init__(self):
        super().__init__()
        self._me = self
        QTimer.singleShot(0, self._finish)

    def _finish(self):
        del self._me  # It can't crash here!
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        QTimer.singleShot(10, QCoreApplication.instance().quit)


if __name__ == '__main__':
    app = QCoreApplication([])
    Dispatcher()
    app.exec()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from enum import IntEnum

from PySide6.QtCore import QAbstractTableModel, QBitArray, Qt, Signal, Slot


class Column(IntEnum):
    NUM_COLUMN = 0
    COILS_COLUMN = 1
    HOLDING_COLUMN = 2
    COLUMN_COUNT = 3
    ROW_COUNT = 10


class WriteRegisterModel(QAbstractTableModel):

    update_viewport = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_coils = QBitArray(Column.ROW_COUNT, False)
        self.m_number = 0
        self.m_address = 0
        self.m_holdingRegisters = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

    def rowCount(self, parent):
        return Column.ROW_COUNT

    def columnCount(self, parent):
        return Column.COLUMN_COUNT

    def data(self, index, role):
        row = index.row()
        column = index.column()
        if not index.isValid() or row >= Column.ROW_COUNT or column >= Column.COLUMN_COUNT:
            return None

        assert self.m_coils.size() == Column.ROW_COUNT
        assert len(self.m_holdingRegisters) == Column.ROW_COUNT

        if column == Column.NUM_COLUMN and role == Qt.ItemDataRole.DisplayRole:
            return f"{row}"

        if column == Column.COILS_COLUMN and role == Qt.ItemDataRole.CheckStateRole:  # coils
            return Qt.CheckState.Checked if self.m_coils[row] else Qt.CheckState.Unchecked

        # holding registers
        if column == Column.HOLDING_COLUMN and role == Qt.ItemDataRole.DisplayRole:
            reg = self.m_holdingRegisters[row]
            return f"0x{reg:x}"
        return None

    def headerData(self, section, orientation, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return None

        if orientation == Qt.Orientation.Horizontal:
            if section == Column.NUM_COLUMN:
                return "#"
            if section == Column.COILS_COLUMN:
                return "Coils "
            if section == Column.HOLDING_COLUMN:
                return "Holding Registers"
        return None

    def setData(self, index, value, role):
        row = index.row()
        column = index.column()
        if not index.isValid() or row >= Column.ROW_COUNT or column >= Column.COLUMN_COUNT:
            return False

        assert self.m_coils.size() == Column.ROW_COUNT
        assert len(self.m_holdingRegisters) == Column.ROW_COUNT

        if column == Column.COILS_COLUMN and role == Qt.ItemDataRole.CheckStateRole:  # coils
            s = Qt.CheckState(int(value))
            if s == Qt.CheckState.Checked:
                self.m_coils.setBit(row)
            else:
                self.m_coils.clearBit(row)
            self.dataChanged.emit(index, index)
            return True

        if column == Column.HOLDING_COLUMN and role == Qt.ItemDataRole.EditRole:
            # holding registers
            base = 16 if value.startswith("0x") else 10
            self.m_holdingRegisters[row] = int(value, base=base)
            self.dataChanged.emit(index, index)
            return True

        return False

    def flags(self, index):
        row = index.row()
        column = index.column()
        flags = super().flags(index)
        if not index.isValid() or row >= Column.ROW_COUNT or column >= Column.COLUMN_COUNT:
            return flags

        if row < self.m_address or row >= (self.m_address + self.m_number):
            flags &= ~Qt.ItemFlag.ItemIsEnabled

        if column == Column.COILS_COLUMN:  # coils
            return flags | Qt.ItemFlag.ItemIsUserCheckable
        if column == Column.HOLDING_COLUMN:  # holding registers
            return flags | Qt.ItemFlag.ItemIsEditable
        return flags

    @Slot(int)
    def set_start_address(self, address):
        self.m_address = address
        self.update_viewport.emit()

    @Slot(str)
    def set_number_of_values(self, number):
        self.m_number = int(number)
        self.update_viewport.emit()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the relationaltablemodel example from Qt v6.x"""

import sys

import connection

from PySide6.QtCore import QObject, Qt
from PySide6.QtSql import (QSqlQuery, QSqlRelation, QSqlRelationalDelegate,
                           QSqlRelationalTableModel)
from PySide6.QtWidgets import QApplication, QTableView


def initializeModel(model):

    model.setTable("employee")
    model.setEditStrategy(QSqlRelationalTableModel.OnManualSubmit)
    model.setRelation(2, QSqlRelation("city", "id", "name"))
    model.setRelation(3, QSqlRelation("country", "id", "name"))
    model.setHeaderData(0, Qt.Orientation.Horizontal, QObject().tr("ID"))

    model.setHeaderData(1, Qt.Orientation.Horizontal, QObject().tr("Name"))
    model.setHeaderData(2, Qt.Orientation.Horizontal, QObject().tr("City"))
    model.setHeaderData(3, Qt.Orientation.Horizontal, QObject().tr("Country"))

    model.select()


def createView(title, model):

    table_view = QTableView()
    table_view.setModel(model)
    table_view.setItemDelegate(QSqlRelationalDelegate(table_view))
    table_view.setWindowTitle(title)

    return table_view


def createRelationalTables():

    query = QSqlQuery()

    query.exec("create table employee(id int primary key, name varchar(20), city int, country int)")
    query.exec("insert into employee values(1, 'Espen', 5000, 47)")
    query.exec("insert into employee values(2, 'Harald', 80000, 49)")
    query.exec("insert into employee values(3, 'Sam', 100, 1)")

    query.exec("create table city(id int, name varchar(20))")
    query.exec("insert into city values(100, 'San Jose')")
    query.exec("insert into city values(5000, 'Oslo')")
    query.exec("insert into city values(80000, 'Munich')")

    query.exec("create table country(id int, name varchar(20))")
    query.exec("insert into country values(1, 'USA')")
    query.exec("insert into country values(47, 'Norway')")
    query.exec("insert into country values(49, 'Germany')")


if __name__ == "__main__":
    app = QApplication(sys.argv)

    connection.createConnection()
    createRelationalTables()

    model = QSqlRelationalTableModel()

    initializeModel(model)

    title = "Relational Table Model"

    window = createView(title, model)
    window.resize(600, 200)
    window.show()

    sys.exit(app.exec())



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/layouts/basiclayout example from Qt v5.x"""

import sys

from PySide6.QtWidgets import (QApplication, QComboBox, QDialog,
                               QDialogButtonBox, QGridLayout, QGroupBox,
                               QFormLayout, QHBoxLayout, QLabel, QLineEdit,
                               QMenu, QMenuBar, QPushButton, QSpinBox,
                               QTextEdit, QVBoxLayout)


class Dialog(QDialog):
    num_grid_rows = 3
    num_buttons = 4

    def __init__(self):
        super().__init__()

        self.create_menu()
        self.create_horizontal_group_box()
        self.create_grid_group_box()
        self.create_form_group_box()

        big_editor = QTextEdit()
        big_editor.setPlainText("This widget takes up all the remaining space "
                                "in the top-level layout.")

        button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok
                                      | QDialogButtonBox.StandardButton.Cancel)

        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)

        main_layout = QVBoxLayout()
        main_layout.setMenuBar(self._menu_bar)
        main_layout.addWidget(self._horizontal_group_box)
        main_layout.addWidget(self._grid_group_box)
        main_layout.addWidget(self._form_group_box)
        main_layout.addWidget(big_editor)
        main_layout.addWidget(button_box)
        self.setLayout(main_layout)

        self.setWindowTitle("Basic Layouts")

    def create_menu(self):
        self._menu_bar = QMenuBar()

        self._file_menu = QMenu("&File", self)
        self._exit_action = self._file_menu.addAction("E&xit")
        self._menu_bar.addMenu(self._file_menu)

        self._exit_action.triggered.connect(self.accept)

    def create_horizontal_group_box(self):
        self._horizontal_group_box = QGroupBox("Horizontal layout")
        layout = QHBoxLayout()

        for i in range(Dialog.num_buttons):
            button = QPushButton(f"Button {i + 1}")
            layout.addWidget(button)

        self._horizontal_group_box.setLayout(layout)

    def create_grid_group_box(self):
        self._grid_group_box = QGroupBox("Grid layout")
        layout = QGridLayout()

        for i in range(Dialog.num_grid_rows):
            label = QLabel(f"Line {i + 1}:")
            line_edit = QLineEdit()
            layout.addWidget(label, i + 1, 0)
            layout.addWidget(line_edit, i + 1, 1)

        self._small_editor = QTextEdit()
        self._small_editor.setPlainText("This widget takes up about two thirds of the grid layout.")

        layout.addWidget(self._small_editor, 0, 2, 4, 1)

        layout.setColumnStretch(1, 10)
        layout.setColumnStretch(2, 20)
        self._grid_group_box.setLayout(layout)

    def create_form_group_box(self):
        self._form_group_box = QGroupBox("Form layout")
        layout = QFormLayout()
        layout.addRow(QLabel("Line 1:"), QLineEdit())
        layout.addRow(QLabel("Line 2, long text:"), QComboBox())
        layout.addRow(QLabel("Line 3:"), QSpinBox())
        self._form_group_box.setLayout(layout)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    dialog = Dialog()
    sys.exit(dialog.exec())



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QFlags'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt, QTemporaryFile, QFile, QIODevice, QObject


class QFlagTest(unittest.TestCase):
    '''Test case for usage of flags'''

    def testCallFunction(self):
        f = QTemporaryFile()
        self.assertTrue(f.open())
        fileName = f.fileName()
        f.close()

        f = QFile(fileName)
        of = (QIODevice.OpenModeFlag.Truncate | QIODevice.OpenModeFlag.Text
              | QIODevice.OpenModeFlag.ReadWrite)
        self.assertEqual(f.open(of), True)
        om = f.openMode()
        self.assertEqual(om & QIODevice.OpenModeFlag.Truncate, QIODevice.OpenModeFlag.Truncate)
        self.assertEqual(om & QIODevice.OpenModeFlag.Text, QIODevice.OpenModeFlag.Text)
        self.assertEqual(om & QIODevice.OpenModeFlag.ReadWrite, QIODevice.OpenModeFlag.ReadWrite)
        expected = (QIODevice.OpenModeFlag.Truncate | QIODevice.OpenModeFlag.Text
                    | QIODevice.OpenModeFlag.ReadWrite)
        self.assertTrue(om == expected)
        f.close()


class QFlagOperatorTest(unittest.TestCase):
    '''Test case for operators in QFlags'''

    def testInvert(self):
        '''QFlags ~ (invert) operator'''
        self.assertEqual(type(~QIODevice.OpenModeFlag.ReadOnly), QIODevice.OpenMode)

    def testOr(self):
        '''QFlags | (or) operator'''
        self.assertEqual(type(QIODevice.OpenModeFlag.ReadOnly | QIODevice.OpenModeFlag.WriteOnly),
                         QIODevice.OpenMode)

    def testAnd(self):
        '''QFlags & (and) operator'''
        self.assertEqual(type(QIODevice.OpenModeFlag.ReadOnly & QIODevice.OpenModeFlag.WriteOnly),
                         QIODevice.OpenMode)

    def testIOr(self):
        '''QFlags |= (ior) operator'''
        flag = Qt.WindowFlags()
        self.assertTrue(Qt.WindowType.Widget == 0)
        self.assertFalse(flag & Qt.WindowType.Widget)
        result = flag & Qt.WindowType.Widget
        self.assertTrue(result == 0)
        flag |= Qt.WindowType.WindowMinimizeButtonHint
        self.assertTrue(flag & Qt.WindowType.WindowMinimizeButtonHint)

    def testInvertOr(self):
        '''QFlags ~ (invert) operator over the result of an | (or) operator'''
        self.assertEqual(type(~(Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEditable)),
                         Qt.ItemFlags)

    def testEqual(self):
        '''QFlags == operator'''
        flags = Qt.WindowType.Window
        flags |= Qt.WindowType.WindowMinimizeButtonHint
        flag_type = (flags & Qt.WindowType.WindowType_Mask)
        self.assertEqual(flag_type, Qt.WindowType.Window)

        self.assertEqual(Qt.KeyboardModifiers(Qt.KeyboardModifier.ControlModifier),
                         Qt.KeyboardModifier.ControlModifier)

    def testOperatorBetweenFlags(self):
        '''QFlags & QFlags'''
        flags = Qt.ItemFlag.NoItemFlags | Qt.ItemFlag.ItemIsUserCheckable
        newflags = Qt.ItemFlag.NoItemFlags | Qt.ItemFlag.ItemIsUserCheckable
        self.assertTrue(flags & newflags)

    def testOperatorDifferentOrder(self):
        '''Different ordering of arguments'''
        flags = Qt.ItemFlag.NoItemFlags | Qt.ItemFlag.ItemIsUserCheckable
        self.assertEqual(flags | Qt.ItemFlag.ItemIsEnabled, Qt.ItemFlag.ItemIsEnabled | flags)

    def testEqualNonNumericalObject(self):
        '''QFlags ==,!= non-numerical object '''
        flags = Qt.ItemFlag.NoItemFlags | Qt.ItemFlag.ItemIsUserCheckable

        self.assertTrue(flags != None)  # noqa: E711
        self.assertFalse(flags == None)  # noqa: E711

        self.assertTrue(flags != "tomato")
        self.assertFalse(flags == "tomato")

        with self.assertRaises(TypeError):
            flags > None
        with self.assertRaises(TypeError):
            flags >= None
        with self.assertRaises(TypeError):
            flags < None
        with self.assertRaises(TypeError):
            flags <= None


class QFlagsOnQVariant(unittest.TestCase):
    def testQFlagsOnQVariant(self):
        o = QObject()
        o.setProperty("foo", QIODevice.OpenModeFlag.ReadOnly | QIODevice.OpenModeFlag.WriteOnly)
        self.assertEqual(type(o.property("foo")), QIODevice.OpenMode)


class QEnumFlagDefault(unittest.TestCase):
    """
        Check that old flag and enum syntax can be used.
        The signatures of these surrogate functions intentionally do not exist
        because people should learn to use the new Enums correctly.
    """
    def testOldQFlag(self):
        self.assertEqual(Qt.AlignmentFlag(), Qt.AlignmentFlag(0))
        oldFlag = Qt.Alignment()
        oldEnum = Qt.AlignmentFlag()
        self.assertEqual(type(oldFlag), Qt.Alignment)
        self.assertEqual(type(oldEnum), Qt.AlignmentFlag)
        self.assertEqual(type(oldFlag), type(oldEnum))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from helper.usesqapplication import UsesQApplication  # noqa: E402, F401

from PySide6.QtCore import QObject, QUrl, Property, qInstallMessageHandler  # noqa: E402
from PySide6.QtQml import ListProperty, QmlElement  # noqa: E402
from PySide6.QtQuick import QQuickView  # noqa: E402


QML_IMPORT_NAME = "test.ListPropertyTest"
QML_IMPORT_MAJOR_VERSION = 1

output_messages = []


def message_handler(mode, context, message):
    global output_messages
    output_messages.append(f"{message}")


class InheritsQObject(QObject):
    pass


def dummyFunc():
    pass


@QmlElement
class Person(QObject):
    def __init__(self, parent=None):
        super().__init__(parent=None)
        self._name = ''
        self._friends = []

    def appendFriend(self, friend):
        self._friends.append(friend)

    def friendCount(self):
        return len(self._friends)

    def friend(self, index):
        return self._friends[index]

    def removeLastItem(self):
        if len(self._friends) > 0:
            self._friends.pop()

    def replace(self, index, friend):
        if 0 <= index < len(self._friends):
            self._friends[index] = friend

    def clear(self):
        self._friends.clear()

    @Property(str, final=True)
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    friends = ListProperty(QObject, append=appendFriend, count=friendCount, at=friend,
                           removeLast=removeLastItem, replace=replace, clear=clear)


class TestListProperty(UsesQApplication):
    def testIt(self):

        # Verify that type checking works properly
        type_check_error = False

        try:
            ListProperty(QObject)
            ListProperty(InheritsQObject)
        except Exception:
            type_check_error = True

        self.assertFalse(type_check_error)

        try:
            ListProperty(int)
        except TypeError:
            type_check_error = True

        self.assertTrue(type_check_error)

        # Verify that method validation works properly
        method_check_error = False

        try:
            ListProperty(QObject, append=None, at=None, count=None, replace=None, clear=None,
                         removeLast=None)  # Explicitly setting None
            ListProperty(QObject, append=dummyFunc)
            ListProperty(QObject, count=dummyFunc, at=dummyFunc)
        except Exception:
            method_check_error = True

        self.assertFalse(method_check_error)

        try:
            ListProperty(QObject, append=QObject())
        except Exception:
            method_check_error = True

        self.assertTrue(method_check_error)

    def testListPropParameters(self):
        global output_messages
        qInstallMessageHandler(message_handler)
        view = QQuickView()
        file = Path(__file__).resolve().parent / 'listproperty.qml'
        self.assertTrue(file.is_file())
        view.setSource(QUrl.fromLocalFile(file))
        view.show()
        self.assertEqual(output_messages[0], "List length: 3")
        self.assertEqual(output_messages[1], "First element: Alice")
        self.assertEqual(output_messages[2], "Removing last item: Charlie")
        self.assertEqual(output_messages[3], "Replacing last item: Bob")
        self.assertEqual(output_messages[4], "Replaced last item: David")
        self.assertEqual(output_messages[5], "List length after clearing: 0")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QTextLayout, QTextOption
from helper.usesqapplication import UsesQApplication


class QTextLineTest(UsesQApplication):

    def testCursorToX(self):
        textLayout = QTextLayout()
        textLayout.beginLayout()
        line = textLayout.createLine()
        self.assertTrue(line.isValid())
        x, cursorPos = line.cursorToX(0)
        self.assertEqual(type(x), float)
        self.assertEqual(type(cursorPos), int)
        x, cursorPos = line.cursorToX(1)
        self.assertEqual(type(x), float)
        self.assertEqual(type(cursorPos), int)

    def testTextOption(self):
        """PYSIDE-2088, large enum values causing MSVC issues."""
        v = QTextOption.Flag.IncludeTrailingSpaces | QTextOption.Flag.ShowTabsAndSpaces
        self.assertEqual(v.value, 2147483649)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import calendar
import datetime
import os
import site
import sys
from pathlib import Path

from build_scripts.options import Options
from build_scripts.utils import (parse_cmake_conf_assignments_by_key,
                                 remove_tree, run_instruction)

options = Options()


class CI:
    def __init__(self):
        # Values must match COIN thrift
        self.HOST_OS = options.option_value("os")
        self.TARGET_OS = options.option_value("targetOs")
        self.HOST_ARCH = options.option_value("hostArch")
        self.TARGET_ARCH = options.option_value("targetArch")
        self.HOST_OS_VER = options.option_value("osVer")
        self.ENV_INSTALL_DIR = options.option_value("instdir")
        self.ENV_AGENT_DIR = options.option_value("agentdir") or "."
        self.COMPILER = options.option_value("compiler")
        self.USE_SCCACHE = options.option_value("compiler-launcher")
        self.INTEGRATION_ID = options.option_value("coinIntegrationId") or str(
            calendar.timegm(datetime.datetime.now().timetuple())
        )
        self.FEATURES = []
        _ci_features = options.option_value("features")
        if _ci_features is not None:
            for f in _ci_features.split(", "):
                self.FEATURES.append(f)
        self.RELEASE_CONF = options.has_option("packaging")
        self.TEST_PHASE = options.option_value("phase")
        if self.TEST_PHASE not in ["ALL", "BUILD"]:
            self.TEST_PHASE = "ALL"


def get_ci_exe_path(ci_install_dir, ci_host_os, qtexe):
    """
    qtexe can only be 'qmake' or 'qtpaths'
    """
    ext = ""
    if ci_host_os == "Windows":
        ext = ".exe"

    _path = Path(ci_install_dir) / "bin" / f"{qtexe}{ext}"

    return f"--{qtexe}={_path}"


def get_env_or_raise(name: str) -> str:
    o = os.getenv(name)
    if o is None:
        raise Exception(f"Variable not defined: {name}")
    return o


def get_qtci_virtualenv(python_ver, log, host, host_arch, target_arch):
    _exe = "python"
    _env = os.environ.get("PYSIDE_VIRTUALENV") or f"env{python_ver}"
    env_python = f"{_env}/bin/python"
    env_pip = f"{_env}/bin/pip"

    if host == "Windows":
        log.info("New virtualenv to build {target_arch} in {host_arch} host")
        _exe = "python.exe"
        if python_ver.startswith("3"):
            var = f"PYTHON{python_ver}-64_PATH"
            log.info(f"Try to find python from {var} env variable")
            _path = Path(os.getenv(var, ""))
            _exe = _path / "python.exe"
            if not _exe.is_file():
                log.warning(f"Can't find python.exe from {_exe}, using default python3")
                _exe = Path(get_env_or_raise("PYTHON3_PATH")) / "python.exe"
        env_python = rf"{_env}\Scripts\python.exe"
        env_pip = rf"{_env}\Scripts\pip.exe"
    else:
        _exe = f"python{python_ver}"
        try:
            run_instruction([_exe, "--version"], f"Failed to guess python version {_exe}")
        except Exception as e:
            print(f"Exception {type(e).__name__}: {e}")
            _exe = "python3"
    return (_exe, _env, env_pip, env_python)


def get_current_script_path():
    """Returns the absolute path containing this script."""
    try:
        this_file = __file__
    except NameError:
        this_file = sys.argv[0]
    this_file = Path(this_file).resolve()
    return this_file.parents[0]


def is_snapshot_build():
    """
    Returns True if project needs to be built with --snapshot-build

    This is true if the version found in .cmake.conf is not a
    pre-release version (no alphas, betas).

    This eliminates the need to remove the --snapshot-build option
    on a per-release branch basis (less things to remember to do
    for a release).
    """
    # This returns pyside-setup/coin/ so we go one level down
    # to get the root of the repo
    setup_script_dir = get_current_script_path()
    pyside_project_dir = setup_script_dir / ".." / "sources" / "pyside6"

    d = parse_cmake_conf_assignments_by_key(str(pyside_project_dir))
    release_version_type = d.get("pyside_PRE_RELEASE_VERSION_TYPE")
    pre_release_version = d.get("pyside_PRE_RELEASE_VERSION")
    if pre_release_version and release_version_type:
        return True
    return False


def get_architecture(ci):
    return "32" if ci.TARGET_ARCH == "X86" else "64"


def get_python_version(ci):
    python_ver = "3"
    if ci.TARGET_OS == "Linux" and ci.HOST_ARCH != "aarch64":
        python_ver = "3.11"
    elif ci.TARGET_OS == "Windows":
        python_ver = "3.10.0"
    return python_ver


def remove_variables(vars):
    for env_var in vars:
        if os.environ.get(env_var):
            del os.environ[env_var]


def setup_virtualenv(python, exe, env, pip, log, ci):
    # Within Ubuntu 24.04 one can't install anything with pip to outside of
    # virtual env. Trust that we already have proper virtualenv installed.
    if os.environ.get("HOST_OSVERSION_COIN") != "ubuntu_24_04":
        virtualenv_version = "20.7.2"
        # 20.7.2 is too old for 3.13
        if sys.version_info[1] > 12:
            virtualenv_version = "20.32.0"
        run_instruction(
            [str(python), "-m", "pip", "install", "--user", "virtualenv==" + virtualenv_version],
            "Failed to pin virtualenv",
        )
    # installing to user base might not be in PATH by default.
    env_path = Path(str(site.USER_BASE)) / "bin"
    v_env = env_path / "virtualenv"
    if sys.platform == "win32":
        if ci.TARGET_ARCH == "aarch64":
            env_path = os.path.join(site.USER_BASE, "Python311-arm64", "Scripts")
        else:
            env_path = os.path.join(site.USER_BASE, "Scripts")
        v_env = os.path.join(env_path, "virtualenv.exe")
    try:
        run_instruction([str(v_env), "--version"], "Using default virtualenv")
    except Exception as e:
        log.info("Failed to use the default virtualenv")
        log.info(f"{type(e).__name__}: {e}")
        v_env = "virtualenv"
    run_instruction([str(v_env), "-p", str(exe), str(env)], "Failed to create virtualenv")
    # Pip is always upgraded when CI template is provisioned,
    # upgrading it in later phase may cause perm issue
    run_instruction(
        [str(pip), "install", "-r", "requirements.txt"], "Failed to install dependencies"
    )


def call_setup(python_ver, ci, phase, log, buildnro=0):
    print("call_setup")
    print("python_ver", python_ver)
    print("phase", phase)
    exe, env, pip, env_python = get_qtci_virtualenv(
        python_ver, log, ci.HOST_OS, ci.HOST_ARCH, ci.TARGET_ARCH
    )

    if phase not in ["BUILD", "TEST"]:
        sys.exit(1)

    remove_tree(env, True)
    # Pinning the virtualenv before creating one
    # Use pip3 if possible while pip seems to install the virtualenv to wrong dir in some OS
    python = "python3"
    if sys.platform == "win32":
        python = Path(get_env_or_raise("PYTHON3_PATH")) / "python.exe"

    if phase == "BUILD":
        setup_virtualenv(python, exe, env, pip, log, ci)
    elif phase == "TEST":

        if ci.HOST_OS == "MacOS" and ci.HOST_ARCH == "ARM64":
            v_env = "virtualenv"
            run_instruction([str(v_env), "-p", str(exe), str(env)], "Failed to create virtualenv")
            run_instruction(
                [pip, "install", "-r", "requirements.txt"], "Failed to install dependencies"
            )
        else:
            setup_virtualenv(python, exe, env, pip, log, ci)
            # Install distro to replace missing platform.linux_distribution() in python3.8
            run_instruction([pip, "install", "distro"], "Failed to install distro")

    if phase == "BUILD":
        cmd = [
            env_python,
            "-u",
            "setup.py",
            "build",
            "--standalone",
            "--unity",
            "--build-tests",
            "--log-level=verbose",
            "--limited-api=yes",
        ]

        if ci.TARGET_ARCH == "X86_64-ARM64":
            cmd += ["--macos-arch='x86_64;arm64'"]

        if ci.USE_SCCACHE:
            cmd += [f"--compiler-launcher={ci.USE_SCCACHE}"]

        if is_snapshot_build():
            cmd += ["--snapshot-build"]

        qtpaths_path = get_ci_exe_path(ci.ENV_INSTALL_DIR, ci.HOST_OS, "qtpaths")
        cmd.append(qtpaths_path)

        # Due to certain older CMake versions generating very long paths
        # (at least with CMake 3.6.2) when using the export() function,
        # pass the shorter paths option on Windows so we don't hit
        # the path character length limit (260).
        if ci.HOST_OS == "Windows":
            cmd += ["--shorter-paths"]

        cmd += ["--package-timestamp=" + ci.INTEGRATION_ID]

        env = os.environ
        run_instruction(cmd, "Failed to run setup.py for build", initial_env=env)
    elif phase == "TEST":
        cmd = [
            env_python,
            "testrunner.py",
            "test",
            "--blacklist",
            "build_history/blacklist.txt",
            f"--buildno={buildnro}",
        ]
        run_instruction(cmd, "Failed to run testrunner.py")

        qmake_path = get_ci_exe_path(ci.ENV_INSTALL_DIR, ci.HOST_OS, "qmake")

        # Try to install built wheels, and build some buildable examples.
        if ci.RELEASE_CONF:
            wheel_tester_path = os.path.join("testing", "wheel_tester.py")
            # Run the test for the new set of wheels
            cmd = [env_python, wheel_tester_path, qmake_path, "--wheels-dir=dist", "--new"]
            run_instruction(cmd, "Error while running wheel_tester.py on new wheels")



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

import numpy
import sys

from PySide6.QtCore import QRangeModel
from PySide6.QtGui import QKeySequence
from PySide6.QtWidgets import QApplication, QListView, QMainWindow, QTableView, QTabWidget


STRING_LIST = ["item1", "item2", "item3", "item4"]
INT_LIST = [1, 2, 3]
INT_TABLE = [[1, 2], [3, 4], [5, 6]]

NP_INT_ARRAY = numpy.array([1, 2, 3], dtype=numpy.int32)
NP_DOUBLE_ARRAY = numpy.array([1.1, 2.2, 3.3], dtype=numpy.double)

NP_INT_TABLE = numpy.array([[1, 2, 3], [4, 5, 6]], dtype=numpy.int32)
NP_DOUBLE_TABLE = numpy.array([[1.1, 2.2, 3.3], [4.4, 5.5, 6.6]], dtype=numpy.double)


def print_numpy_data():
    print("--------------------------------")
    print("NP_INT_ARRAY=", NP_INT_ARRAY)
    print("NP_INT_TABLE=", NP_INT_TABLE)
    print("NP_DOUBLE_ARRAY=", NP_DOUBLE_ARRAY)
    print("NP_DOUBLE_TABLE=", NP_DOUBLE_TABLE)
    print("---------------------------------\n")


if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = QMainWindow()
    window.setMinimumWidth(600)
    file_menu = window.menuBar().addMenu("File")
    file_menu.addAction("Output numpy data", print_numpy_data)
    file_menu.addAction("Quit", QKeySequence(QKeySequence.StandardKey.Quit), window.close)
    help_menu = window.menuBar().addMenu("Help")
    help_menu.addAction("About Qt", app.aboutQt)

    tab_widget = QTabWidget()
    window.setCentralWidget(tab_widget)

    list_view = QListView()
    model = QRangeModel(STRING_LIST)
    list_view.setModel(model)
    tab_widget.addTab(list_view, "Python String List")

    list_view = QListView()
    model = QRangeModel(INT_LIST)
    list_view.setModel(model)
    tab_widget.addTab(list_view, "Python int List")

    table_view = QTableView()
    model = QRangeModel(INT_TABLE)
    table_view.setModel(model)
    tab_widget.addTab(table_view, "Python Int Table")

    list_view = QListView()
    model = QRangeModel(NP_INT_ARRAY)
    list_view.setModel(model)
    tab_widget.addTab(list_view, "Numpy Int List")

    list_view = QListView()
    model = QRangeModel(NP_DOUBLE_ARRAY)
    list_view.setModel(model)
    tab_widget.addTab(list_view, "Numpy Double List")

    table_view = QTableView()
    model = QRangeModel(NP_INT_TABLE)
    table_view.setModel(model)
    tab_widget.addTab(table_view, "Numpy Int Table")

    table_view = QTableView()
    model = QRangeModel(NP_DOUBLE_TABLE)
    table_view.setModel(model)
    tab_widget.addTab(table_view, "Numpy Double Table")

    window.setWindowTitle("QRangeModel")
    window.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QTimer, Qt
from PySide6.QtWidgets import QListWidget, QListWidgetItem
from helper.usesqapplication import UsesQApplication


class QListWidgetTest(UsesQApplication):

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def populateList(self, lst):
        o = QObject()
        o.setObjectName("obj")

        item = QListWidgetItem("item0")
        item.setData(Qt.ItemDataRole.UserRole, o)
        # item._data = o
        self.assertTrue(sys.getrefcount(o), 3)
        self.assertTrue(sys.getrefcount(item), 2)
        lst.addItem(item)
        self.assertTrue(sys.getrefcount(item), 3)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def checkCurrentItem(self, lst):
        item = lst.currentItem()
        self.assertTrue(sys.getrefcount(item), 3)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def checkItemData(self, lst):
        item = lst.currentItem()
        o = item.data(Qt.UserRole)
        self.assertTrue(sys.getrefcount(o), 4)
        self.assertEqual(o, item._data)
        self.assertTrue(sys.getrefcount(o), 2)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testConstructorWithParent(self):
        lst = QListWidget()
        self.populateList(lst)
        self.checkCurrentItem(lst)
        i = lst.item(0)
        self.assertTrue(sys.getrefcount(i), 3)

        del lst
        self.assertTrue(sys.getrefcount(i), 2)
        del i

    def testIt(self):
        lst = QListWidget()
        lst.show()
        slot = lambda: lst.removeItemWidget(lst.currentItem())  # noqa: E731
        lst.addItem(QListWidgetItem("foo"))
        QTimer.singleShot(0, slot)
        QTimer.singleShot(0, lst.close)
        self.app.exec()
        self.assertEqual(lst.count(), 1)

    def testClear(self):
        lst = QListWidget()
        lst.addItem("foo")
        item = lst.item(0)
        self.assertIsNone(lst.clear())
        self.assertRaises(RuntimeError, lambda: item.text())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import Slot
from PySide6.QtGui import QIntValidator
from PySide6.QtWidgets import QComboBox
from PySide6.QtSerialPort import QSerialPort, QSerialPortInfo

from PySide6.QtWidgets import QDialog

from ui_settingsdialog import Ui_SettingsDialog


BLANK_STRING = "N/A"


CUSTOM_BAUDRATE_INDEX = 4


class Settings():

    def __init__(self):
        self.name = ""
        self.baud_rate = 0
        self.string_baud_rate = ""
        self.data_bits = QSerialPort.DataBits.Data8
        self.string_data_bits = ""
        self.parity = QSerialPort.Parity.NoParity
        self.string_parity = ""
        self.stop_bits = QSerialPort.StopBits.OneStop
        self.string_stop_bits = ""
        self.flow_control = QSerialPort.FlowControl.SoftwareControl
        self.string_flow_control = ""
        self.local_echo_enabled = False


class SettingsDialog(QDialog):

    def __init__(self, parent=None):
        super().__init__(parent)

        self.m_ui = Ui_SettingsDialog()
        self._custom_port_index = -1
        self.m_ui.setupUi(self)
        self.m_currentSettings = Settings()
        self.m_intValidator = QIntValidator(0, 4000000, self)

        self.m_ui.baudRateBox.setInsertPolicy(QComboBox.InsertPolicy.NoInsert)

        self.m_ui.applyButton.clicked.connect(self.apply)
        self.m_ui.serialPortInfoListBox.currentIndexChanged.connect(self.show_port_info)
        self.m_ui.baudRateBox.currentIndexChanged.connect(self.check_custom_baud_rate_policy)
        self.m_ui.serialPortInfoListBox.currentIndexChanged.connect(
            self.check_custom_device_path_policy)

        self.fill_ports_parameters()
        self.fill_ports_info()

        self.update_settings()

    def settings(self):
        return self.m_currentSettings

    @Slot(int)
    def show_port_info(self, idx):
        if idx == -1:
            return

        list = self.m_ui.serialPortInfoListBox.itemData(idx)
        count = len(list) if list else 0
        description = list[1] if count > 1 else BLANK_STRING
        self.m_ui.descriptionLabel.setText(f"Description: {description}")
        manufacturer = list[2] if count > 2 else BLANK_STRING
        self.m_ui.manufacturerLabel.setText(f"Manufacturer: {manufacturer}")
        serialno = list[3] if count > 3 else BLANK_STRING
        self.m_ui.serialNumberLabel.setText(f"Serial number: {serialno}")
        location = list[4] if count > 4 else BLANK_STRING
        self.m_ui.locationLabel.setText(f"Location: {location}")
        vendor = list[5] if count > 5 else BLANK_STRING
        self.m_ui.vidLabel.setText(f"Vendor Identifier: {vendor}")
        id = list[6] if count > 6 else BLANK_STRING
        self.m_ui.pidLabel.setText(f"Product Identifier: {id}")

    @Slot()
    def apply(self):
        self.update_settings()
        self.hide()

    @Slot(int)
    def check_custom_baud_rate_policy(self, idx):
        is_custom_baud_rate = idx == CUSTOM_BAUDRATE_INDEX
        self.m_ui.baudRateBox.setEditable(is_custom_baud_rate)
        if is_custom_baud_rate:
            self.m_ui.baudRateBox.clearEditText()
            edit = self.m_ui.baudRateBox.lineEdit()
            edit.setValidator(self.m_intValidator)

    @Slot(int)
    def check_custom_device_path_policy(self, idx):
        is_custom_path = idx == self._custom_port_index
        self.m_ui.serialPortInfoListBox.setEditable(is_custom_path)
        if is_custom_path:
            self.m_ui.serialPortInfoListBox.clearEditText()

    def fill_ports_parameters(self):
        self.m_ui.baudRateBox.addItem("9600", QSerialPort.BaudRate.Baud9600)
        self.m_ui.baudRateBox.addItem("19200", QSerialPort.BaudRate.Baud19200)
        self.m_ui.baudRateBox.addItem("38400", QSerialPort.BaudRate.Baud38400)
        self.m_ui.baudRateBox.addItem("115200", QSerialPort.BaudRate.Baud115200)
        self.m_ui.baudRateBox.addItem("Custom")

        self.m_ui.dataBitsBox.addItem("5", QSerialPort.DataBits.Data5)
        self.m_ui.dataBitsBox.addItem("6", QSerialPort.DataBits.Data6)
        self.m_ui.dataBitsBox.addItem("7", QSerialPort.DataBits.Data7)
        self.m_ui.dataBitsBox.addItem("8", QSerialPort.DataBits.Data8)
        self.m_ui.dataBitsBox.setCurrentIndex(3)

        self.m_ui.parityBox.addItem("None", QSerialPort.Parity.NoParity)
        self.m_ui.parityBox.addItem("Even", QSerialPort.Parity.EvenParity)
        self.m_ui.parityBox.addItem("Odd", QSerialPort.Parity.OddParity)
        self.m_ui.parityBox.addItem("Mark", QSerialPort.Parity.MarkParity)
        self.m_ui.parityBox.addItem("Space", QSerialPort.Parity.SpaceParity)

        self.m_ui.stopBitsBox.addItem("1", QSerialPort.StopBits.OneStop)
        if sys.platform == "win32":
            self.m_ui.stopBitsBox.addItem("1.5", QSerialPort.StopBits.OneAndHalfStop)

        self.m_ui.stopBitsBox.addItem("2", QSerialPort.StopBits.TwoStop)

        self.m_ui.flowControlBox.addItem("None", QSerialPort.FlowControl.NoFlowControl)
        self.m_ui.flowControlBox.addItem("RTS/CTS", QSerialPort.FlowControl.HardwareControl)
        self.m_ui.flowControlBox.addItem("XON/XOFF", QSerialPort.FlowControl.SoftwareControl)

    def fill_ports_info(self):
        self.m_ui.serialPortInfoListBox.clear()
        for info in QSerialPortInfo.availablePorts():
            list = []
            description = info.description()
            manufacturer = info.manufacturer()
            serial_number = info.serialNumber()
            list.append(info.portName())
            list.append(description if description else BLANK_STRING)
            list.append(manufacturer if manufacturer else BLANK_STRING)
            list.append(serial_number if serial_number else BLANK_STRING)
            list.append(info.systemLocation())
            vid = info.vendorIdentifier()
            list.append(f"{vid:x}" if vid else BLANK_STRING)
            pid = info.productIdentifier()
            list.append(f"{pid:x}" if pid else BLANK_STRING)
            self.m_ui.serialPortInfoListBox.addItem(list[0], list)

        self._custom_port_index = self.m_ui.serialPortInfoListBox.count()
        self.m_ui.serialPortInfoListBox.addItem("Custom")

    def update_settings(self):
        self.m_currentSettings.name = self.m_ui.serialPortInfoListBox.currentText()

        baud_index = self.m_ui.baudRateBox.currentIndex()
        if baud_index == CUSTOM_BAUDRATE_INDEX:
            text = self.m_ui.baudRateBox.currentText()
            self.m_currentSettings.baud_rate = int(text)
        else:
            self.m_currentSettings.baud_rate = self.m_ui.baudRateBox.currentData()
        self.m_currentSettings.string_baud_rate = f"{self.m_currentSettings.baud_rate}"

        self.m_currentSettings.data_bits = self.m_ui.dataBitsBox.currentData()
        self.m_currentSettings.string_data_bits = self.m_ui.dataBitsBox.currentText()

        self.m_currentSettings.parity = self.m_ui.parityBox.currentData()
        self.m_currentSettings.string_parity = self.m_ui.parityBox.currentText()

        self.m_currentSettings.stop_bits = self.m_ui.stopBitsBox.currentData()
        self.m_currentSettings.string_stop_bits = self.m_ui.stopBitsBox.currentText()

        self.m_currentSettings.flow_control = self.m_ui.flowControlBox.currentData()
        self.m_currentSettings.string_flow_control = self.m_ui.flowControlBox.currentText()

        self.m_currentSettings.local_echo_enabled = self.m_ui.localEchoCheckBox.isChecked()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtWidgets import QApplication

from dialog import Dialog

if __name__ == "__main__":
    app = QApplication(sys.argv)

    dialog = Dialog()
    dialog.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtCore import QTimer
from PySide6.QtGui import QPainter
from PySide6.QtWidgets import QWidget


class MyWidget(QWidget):
    def __init__(self):
        super().__init__()
        self._info = None

    def paintEvent(self, e):
        with QPainter(self) as p:
            self._info = p.fontInfo()
        QTimer.singleShot(0, qApp.quit)  # noqa: F821


class TestQPainter(UsesQApplication):
    def testFontInfo(self):
        w = MyWidget()
        w.show()
        self.app.exec()
        self.assertTrue(w._info)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QRectF
from PySide6.QtWidgets import QGraphicsItem
from PySide6.QtGui import QColor
from helper.usesqapplication import UsesQApplication


class Item(QGraphicsItem):

    def __init__(self):
        super().__init__()

    def boundingRect(self):
        return QRectF(0, 0, 100, 100)

    def paint(self, painter, option, widget):
        painter.setBrush(QColor(255, 255, 255))
        painter.drawRect(0, 0, 100, 100)


class QGraphicsViewIsBlockedTest(UsesQApplication):

    def testIsBlockedByModalPanel(self):
        (first, second) = Item().isBlockedByModalPanel()
        self.assertFalse(first)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from PySide6.QtWidgets import QMainWindow, QWidget, QVBoxLayout
from folder.label_in_folder import LabelInFolder
from subproject.subproject_button import SubprojectButton


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Main Window")

        self.central_layout = QVBoxLayout()
        self.central_widget = QWidget()
        self.setCentralWidget(self.central_widget)
        self.central_widget.setLayout(self.central_layout)

        self.label_in_folder = LabelInFolder()
        self.central_layout.addWidget(self.label_in_folder)

        self.subproject_button = SubprojectButton()
        self.central_layout.addWidget(self.subproject_button)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Qt Graphs widgetgallery example from Qt v6.x"""

import sys

from PySide6.QtCore import QSize
from PySide6.QtWidgets import QApplication, QTabWidget

from bargraph import BarGraph
from scattergraph import ScatterGraph
from surfacegraph import SurfaceGraph


class MainWidget(QTabWidget):
    """Tab widget for creating own tabs for Q3DBars, Q3DScatter, and Q3DSurface"""

    def __init__(self, p=None):
        super().__init__(p)

        screen_size = self.screen().size()
        minimum_graph_size = QSize(screen_size.width() / 2, screen_size.height() / 1.75)

        # Create bar graph
        self._bars = BarGraph(minimum_graph_size, screen_size)
        # Create scatter graph
        self._scatter = ScatterGraph(minimum_graph_size, screen_size)
        # Create surface graph
        self._surface = SurfaceGraph(minimum_graph_size, screen_size)

        # Add bars widget
        self.addTab(self._bars.barsWidget(), "Bar Graph")
        # Add scatter widget
        self.addTab(self._scatter.scatterWidget(), "Scatter Graph")
        # Add surface widget
        self.addTab(self._surface.surfaceWidget(), "Surface Graph")


if __name__ == "__main__":
    app = QApplication(sys.argv)

    tabWidget = MainWidget()
    tabWidget.setWindowTitle("Widget Gallery")

    tabWidget.show()
    exit_code = app.exec()
    del tabWidget
    sys.exit(exit_code)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Tests calling NoImplicitConversion using a ExtendsNoImplicitConversion parameter,
   being that the latter defines a new conversion operator for the former, and this one
   has no implicit conversions.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import NoImplicitConversion
from other import ExtendsNoImplicitConversion


class ConversionOperatorForClassWithoutImplicitConversionsTest(unittest.TestCase):
    '''Tests calling NoImplicitConversion constructor using a
       ExtendsNoImplicitConversion parameter.'''

    def testNoImplicitConversion(self):
        '''Basic test to see if the NoImplicitConversion is Ok.'''
        obj = NoImplicitConversion(123)
        # NoImplicitConversion.receivesNoImplicitConversionByValue(NoImplicitConversion)
        self.assertEqual(obj.objId(), NoImplicitConversion.receivesNoImplicitConversionByValue(obj))
        # NoImplicitConversion.receivesNoImplicitConversionByPointer(NoImplicitConversion*)
        self.assertEqual(obj.objId(),
                         NoImplicitConversion.receivesNoImplicitConversionByPointer(obj))
        # NoImplicitConversion.receivesNoImplicitConversionByReference(NoImplicitConversion&)
        self.assertEqual(obj.objId(),
                         NoImplicitConversion.receivesNoImplicitConversionByReference(obj))

    def testPassingExtendsNoImplicitConversionAsNoImplicitConversionByValue(self):
        '''Gives an ExtendsNoImplicitConversion object to a function expecting a
           NoImplicitConversion, passing by value.'''
        obj = ExtendsNoImplicitConversion(123)
        self.assertEqual(obj.objId(), NoImplicitConversion.receivesNoImplicitConversionByValue(obj))

    def testPassingExtendsNoImplicitConversionAsNoImplicitConversionByReference(self):
        '''Gives an ExtendsNoImplicitConversion object to a function expecting a
           NoImplicitConversion, passing by reference.'''
        obj = ExtendsNoImplicitConversion(123)
        self.assertEqual(obj.objId(),
                         NoImplicitConversion.receivesNoImplicitConversionByReference(obj))

    def testPassingExtendsNoImplicitConversionAsNoImplicitConversionByPointer(self):
        '''Gives an ExtendsNoImplicitConversion object to a function expecting
           a NoImplicitConversion, passing by pointer. This should not be
           accepted, since pointers should not be converted.'''
        obj = ExtendsNoImplicitConversion(123)
        self.assertRaises(TypeError,
                          NoImplicitConversion.receivesNoImplicitConversionByPointer, obj)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import struct

from enum import IntEnum

from PySide6.QtBluetooth import (QLowEnergyCharacteristic,
                                 QLowEnergyController,
                                 QLowEnergyDescriptor,
                                 QLowEnergyService,
                                 QBluetoothUuid)
from PySide6.QtQml import QmlElement
from PySide6.QtCore import (QByteArray, QDateTime, QRandomGenerator, QTimer,
                            Property, Signal, Slot, QEnum)

from bluetoothbaseclass import BluetoothBaseClass
from heartrate_global import simulator


# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "HeartRateGame"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class DeviceHandler(BluetoothBaseClass):

    @QEnum
    class AddressType(IntEnum):
        PUBLIC_ADDRESS = 1
        RANDOM_ADDRESS = 2

    measuringChanged = Signal()
    aliveChanged = Signal()
    statsChanged = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)

        self.m_control = None
        self.m_service = None
        self.m_notificationDesc = QLowEnergyDescriptor()
        self.m_currentDevice = None

        self.m_foundHeartRateService = False
        self.m_measuring = False
        self.m_currentValue = 0
        self.m_min = 0
        self.m_max = 0
        self.m_sum = 0
        self.m_avg = 0.0
        self.m_calories = 0.0

        self.m_start = QDateTime()
        self.m_stop = QDateTime()

        self.m_measurements = []
        self.m_addressType = QLowEnergyController.RemoteAddressType.PublicAddress

        self.m_demoTimer = QTimer()

        if simulator():
            self.m_demoTimer.setSingleShot(False)
            self.m_demoTimer.setInterval(2000)
            self.m_demoTimer.timeout.connect(self.updateDemoHR)
            self.m_demoTimer.start()
            self.updateDemoHR()

    def addressType(self):
        if self.m_addressType == QLowEnergyController.RemoteAddressType.RandomAddress:
            return DeviceHandler.AddressType.RANDOM_ADDRESS
        return DeviceHandler.AddressType.PUBLIC_ADDRESS

    @Slot(int)
    def setAddressType(self, type):
        if type == DeviceHandler.AddressType.PUBLIC_ADDRESS:
            self.m_addressType = QLowEnergyController.RemoteAddressType.PublicAddress
        elif type == DeviceHandler.AddressType.RANDOM_ADDRESS:
            self.m_addressType = QLowEnergyController.RemoteAddressType.RandomAddress

    @Slot()
    def resetAddressType(self):
        self.m_addressType = QLowEnergyController.RemoteAddressType.PublicAddress

    @Slot(QLowEnergyController.Error)
    def controllerErrorOccurred(self, device):
        self.error = "Cannot connect to remote device."
        self.icon = BluetoothBaseClass.IconType.IconError

    @Slot()
    def controllerConnected(self):
        self.info = "Controller connected. Search services..."
        self.icon = BluetoothBaseClass.IconType.IconProgress
        self.m_control.discoverServices()

    @Slot()
    def controllerDisconnected(self):
        self.error = "LowEnergy controller disconnected"
        self.icon = BluetoothBaseClass.IconType.IconError

    def setDevice(self, device):
        self.clearMessages()
        self.m_currentDevice = device

        if simulator():
            self.info = "Demo device connected."
            self.icon = BluetoothBaseClass.IconType.IconBluetooth
            return

        # Disconnect and delete old connection
        if self.m_control:
            self.m_control.disconnectFromDevice()
            self.m_control = None

        # Create new controller and connect it if device available
        if self.m_currentDevice:

            # Make connections
#! [Connect-Signals-1]
            self.m_control = QLowEnergyController.createCentral(self.m_currentDevice.device(), self)
#! [Connect-Signals-1]
            self.m_control.setRemoteAddressType(self.m_addressType)
#! [Connect-Signals-2]

            self.m_control.serviceDiscovered.connect(self.serviceDiscovered)
            self.m_control.discoveryFinished.connect(self.serviceScanDone)

            self.m_control.errorOccurred.connect(self.controllerErrorOccurred)
            self.m_control.connected.connect(self.controllerConnected)
            self.m_control.disconnected.connect(self.controllerDisconnected)

            # Connect
            self.m_control.connectToDevice()
#! [Connect-Signals-2]

    @Slot()
    def startMeasurement(self):
        if self.alive:
            self.m_start = QDateTime.currentDateTime()
            self.m_min = 0
            self.m_max = 0
            self.m_avg = 0
            self.m_sum = 0
            self.m_calories = 0.0
            self.m_measuring = True
            self.m_measurements.clear()
            self.measuringChanged.emit()

    @Slot()
    def stopMeasurement(self):
        self.m_measuring = False
        self.measuringChanged.emit()

#! [Filter HeartRate service 1]
    @Slot(QBluetoothUuid)
    def serviceDiscovered(self, gatt):
        if gatt == QBluetoothUuid(QBluetoothUuid.ServiceClassUuid.HeartRate):
            self.info = "Heart Rate service discovered. Waiting for service scan to be done..."
            self.icon = BluetoothBaseClass.IconType.IconProgress
            self.m_foundHeartRateService = True

#! [Filter HeartRate service 1]

    @Slot()
    def serviceScanDone(self):
        self.info = "Service scan done."
        self.icon = BluetoothBaseClass.IconType.IconProgress

        # Delete old service if available
        if self.m_service:
            self.m_service = None

#! [Filter HeartRate service 2]
        # If heartRateService found, create new service
        if self.m_foundHeartRateService:
            self.m_service = self.m_control.createServiceObject(
                QBluetoothUuid(QBluetoothUuid.ServiceClassUuid.HeartRate), self)

        if self.m_service:
            self.m_service.stateChanged.connect(self.serviceStateChanged)
            self.m_service.characteristicChanged.connect(self.updateHeartRateValue)
            self.m_service.descriptorWritten.connect(self.confirmedDescriptorWrite)
            self.m_service.discoverDetails()
        else:
            self.error = "Heart Rate Service not found."
            self.icon = BluetoothBaseClass.IconType.IconError

#! [Filter HeartRate service 2]

# Service functions
#! [Find HRM characteristic]
    @Slot(QLowEnergyService.ServiceState)
    def serviceStateChanged(self, switch):
        if switch == QLowEnergyService.RemoteServiceDiscovering:
            self.info = "Discovering services..."
            self.icon = BluetoothBaseClass.IconType.IconProgress
        elif switch == QLowEnergyService.RemoteServiceDiscovered:
            self.info = "Service discovered."
            self.icon = BluetoothBaseClass.IconType.IconBluetooth
            hrChar = self.m_service.characteristic(
                QBluetoothUuid(QBluetoothUuid.CharacteristicType.HeartRateMeasurement))
            if hrChar.isValid():
                self.m_notificationDesc = hrChar.descriptor(
                    QBluetoothUuid.DescriptorType.ClientCharacteristicConfiguration)
                if self.m_notificationDesc.isValid():
                    self.m_service.writeDescriptor(self.m_notificationDesc,
                                                   QByteArray.fromHex(b"0100"))
            else:
                self.error = "HR Data not found."
                self.icon = BluetoothBaseClass.IconType.IconError
        self.aliveChanged.emit()
#! [Find HRM characteristic]

#! [Reading value]
    @Slot(QLowEnergyCharacteristic, QByteArray)
    def updateHeartRateValue(self, c, value):
        # ignore any other characteristic change. Shouldn't really happen though
        if c.uuid() != QBluetoothUuid(QBluetoothUuid.CharacteristicType.HeartRateMeasurement):
            return

        data = value.data()
        flags = int(data[0])
        # Heart Rate
        hrvalue = 0
        if flags & 0x1:  # HR 16 bit little endian? otherwise 8 bit
            hrvalue = struct.unpack("<H", data[1:3])[0]
        else:
            hrvalue = struct.unpack("B", data[1:2])[0]

        self.addMeasurement(hrvalue)

#! [Reading value]
    @Slot()
    def updateDemoHR(self):
        randomValue = 0
        if self.m_currentValue < 30:  # Initial value
            randomValue = 55 + QRandomGenerator.global_().bounded(30)
        elif not self.m_measuring:  # Value when relax
            random = QRandomGenerator.global_().bounded(5)
            randomValue = self.m_currentValue - 2 + random
            randomValue = max(min(randomValue, 55), 75)
        else:  # Measuring
            random = QRandomGenerator.global_().bounded(10)
            randomValue = self.m_currentValue + random - 2

        self.addMeasurement(randomValue)

    @Slot(QLowEnergyCharacteristic, QByteArray)
    def confirmedDescriptorWrite(self, d, value):
        if (d.isValid() and d == self.m_notificationDesc
                and value == QByteArray.fromHex(b"0000")):
            # disabled notifications . assume disconnect intent
            self.m_control.disconnectFromDevice()
            self.m_service = None

    @Slot()
    def disconnectService(self):
        self.m_foundHeartRateService = False

        # disable notifications
        if (self.m_notificationDesc.isValid() and self.m_service
                and self.m_notificationDesc.value() == QByteArray.fromHex(b"0100")):
            self.m_service.writeDescriptor(self.m_notificationDesc,
                                           QByteArray.fromHex(b"0000"))
        else:
            if self.m_control:
                self.m_control.disconnectFromDevice()
            self.m_service = None

    @Property(bool, notify=measuringChanged)
    def measuring(self):
        return self.m_measuring

    @Property(bool, notify=aliveChanged)
    def alive(self):
        if simulator():
            return True
        if self.m_service:
            return self.m_service.state() == QLowEnergyService.RemoteServiceDiscovered
        return False

    @Property(int, notify=statsChanged)
    def hr(self):
        return self.m_currentValue

    @Property(int, notify=statsChanged)
    def time(self):
        return self.m_start.secsTo(self.m_stop)

    @Property(int, notify=statsChanged)
    def maxHR(self):
        return self.m_max

    @Property(int, notify=statsChanged)
    def minHR(self):
        return self.m_min

    @Property(float, notify=statsChanged)
    def average(self):
        return self.m_avg

    @Property(float, notify=statsChanged)
    def calories(self):
        return self.m_calories

    def addMeasurement(self, value):
        self.m_currentValue = value

        # If measuring and value is appropriate
        if self.m_measuring and value > 30 and value < 250:
            self.m_stop = QDateTime.currentDateTime()
            self.m_measurements.append(value)

            self.m_min = value if self.m_min == 0 else min(value, self.m_min)
            self.m_max = max(value, self.m_max)
            self.m_sum += value
            self.m_avg = float(self.m_sum) / len(self.m_measurements)
            self.m_calories = ((-55.0969 + (0.6309 * self.m_avg) + (0.1988 * 94)
                                + (0.2017 * 24)) / 4.184) * 60 * self.time / 3600

        self.statsChanged.emit()

    addressType = Property(int, addressType, setAddressType, freset=resetAddressType)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectTypeByValue


class ObjectTypeByValueTest (unittest.TestCase):
    def testIt(self):
        factory = ObjectTypeByValue()
        obj = factory.returnSomeKindOfMe()
        # This should crash!
        obj.prop.protectedValueTypeProperty.setX(1.0)
        # just to make sure it will segfault
        obj.prop.protectedValueTypeProperty.setY(2.0)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QWidget, QMainWindow
from PySide6.QtGui import QGuiApplication
from helper.usesqapplication import UsesQApplication


class QWidgetInherit(QMainWindow):
    def __init__(self):
        QWidget.__init__(self)  # Intended: Initialize QWidget instead of base


class NativeEventTestWidget(QWidget):

    nativeEventCount = 0

    def __init__(self):
        super().__init__()

    def nativeEvent(self, eventType, message):
        self.nativeEventCount = self.nativeEventCount + 1
        return [False, 0]


class QWidgetTest(UsesQApplication):

    def testInheritance(self):
        self.assertRaises(TypeError, QWidgetInherit)


class QWidgetVisible(UsesQApplication):

    def testBasic(self):
        # Also related to bug #244, on existence of setVisible'''
        widget = NativeEventTestWidget()
        self.assertTrue(not widget.isVisible())
        widget.setVisible(True)
        self.assertTrue(widget.isVisible())
        self.assertTrue(widget.winId() != 0)
        # skip this test on macOS/Wayland since no native events are received
        if sys.platform == 'darwin' or QGuiApplication.platformName() == 'wayland':
            return
        for i in range(100):
            if widget.nativeEventCount > 0:
                break
            self.app.processEvents()
        self.assertTrue(widget.nativeEventCount > 0)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Point


class TestNewDivision(unittest.TestCase):

    def testIt(self):
        p = Point(4, 4)
        p2 = p / 2
        self.assertEqual(p2, Point(2, 2))


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Helper classes and functions'''

import os
from random import randint


def adjust_filename(filename, orig_mod_filename):
    dirpath = os.path.dirname(os.path.abspath(orig_mod_filename))
    return os.path.join(dirpath, filename)


def _join_qml_errors(errors):
    '''Return an error string from a list of QQmlError'''
    result = ''
    for i, error in enumerate(errors):
        if i:
            result += ', '
        result += error.toString()
    return result


def quickview_errorstring(quickview):
    '''Return an error string from a QQuickView'''
    return _join_qml_errors(quickview.errors())


def qmlcomponent_errorstring(component):
    '''Return an error string from a QQmlComponent'''
    return _join_qml_errors(component.errors())


def random_string(size=5):
    '''Generate random string with the given size'''
    return ''.join(map(chr, [randint(33, 126) for x in range(size)]))



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
import sys
import math
from PySide6.QtWidgets import QApplication, QMainWindow
from PySide6.QtCore import Qt, QPointF, QRandomGenerator
from PySide6.QtCharts import QChart, QLineSeries
from PySide6.QtGui import QPainter

import chartview
import chart

if __name__ == "__main__":

    app = QApplication(sys.argv)

    series = QLineSeries()

    points = [
        QPointF(float(i), math.sin(math.pi / 50 * i) * 100 + QRandomGenerator.global_().bounded(20))
        for i in range(500)]

    series.append(points)

    line_chart = chart.Chart()
    line_chart.addSeries(series)
    line_chart.setTitle("Zoom in/out example")
    line_chart.setAnimationOptions(QChart.SeriesAnimations)
    line_chart.legend().hide()
    line_chart.createDefaultAxes()

    chart_view = chartview.ChartView(line_chart)
    chart_view.setRenderHint(QPainter.Antialiasing, True)

    window = QMainWindow()
    window.setCentralWidget(chart_view)
    window.resize(400, 300)
    window.grabGesture(Qt.PanGesture)
    window.grabGesture(Qt.PinchGesture)
    window.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QT_TR_NOOP, QT_TR_NOOP_UTF8
from PySide6.QtCore import QT_TRANSLATE_NOOP, QT_TRANSLATE_NOOP3, QT_TRANSLATE_NOOP_UTF8


REF_COUNT_DELTA = 0 if sys.version_info >= (3, 14) else 1  # 3.14 borrows references


class QtTrNoopTest(unittest.TestCase):

    def setUp(self):
        self.txt = 'Cthulhu fhtag!'

    def tearDown(self):
        del self.txt
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testQtTrNoop(self):
        refcnt = sys.getrefcount(self.txt)
        result = QT_TR_NOOP(self.txt)
        self.assertEqual(result, self.txt)
        self.assertEqual(sys.getrefcount(result), refcnt + REF_COUNT_DELTA)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testQtTrNoopUtf8(self):
        refcnt = sys.getrefcount(self.txt)
        result = QT_TR_NOOP_UTF8(self.txt)
        self.assertEqual(result, self.txt)
        self.assertEqual(sys.getrefcount(result), refcnt + REF_COUNT_DELTA)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testQtTranslateNoop(self):
        refcnt = sys.getrefcount(self.txt)
        result = QT_TRANSLATE_NOOP(None, self.txt)
        self.assertEqual(result, self.txt)
        self.assertEqual(sys.getrefcount(result), refcnt + REF_COUNT_DELTA)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testQtTranslateNoopUtf8(self):
        refcnt = sys.getrefcount(self.txt)
        result = QT_TRANSLATE_NOOP_UTF8(self.txt)
        self.assertEqual(result, self.txt)
        self.assertEqual(sys.getrefcount(result), refcnt + REF_COUNT_DELTA)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testQtTranslateNoop3(self):
        refcnt = sys.getrefcount(self.txt)
        result = QT_TRANSLATE_NOOP3(None, self.txt, None)
        self.assertEqual(result, self.txt)
        self.assertEqual(sys.getrefcount(result), refcnt + REF_COUNT_DELTA)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from PySide6.QtGui import QGuiApplication
from PySide6.QtQuick import QQuickWindow, QSGRendererInterface

from window_singlethreaded import WindowSingleThreaded


if __name__ == "__main__":
    app = QGuiApplication(sys.argv)
    # only functional when Qt Quick is also using OpenGL
    QQuickWindow.setGraphicsApi(QSGRendererInterface.GraphicsApi.OpenGLRhi)
    window = WindowSingleThreaded()
    window.resize(1024, 768)
    window.show()
    ex = app.exec()
    del window
    sys.exit(ex)



# Copyright (C) 2010 Hans-Peter Jansen <hpj@urpla.net>
# Copyright (C) 2011 Arun Srinivasan <rulfzid@gmail.com>
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import (QWidget)
from PySide6.QtGui import (QPainter)
from PySide6.QtCore import Signal

from starrating import StarRating


class StarEditor(QWidget):
    """ The custom editor for editing StarRatings. """

    # A signal to tell the delegate when we've finished editing.
    editing_finished = Signal()

    def __init__(self, parent=None):
        """ Initialize the editor object, making sure we can watch mouse
            events.
        """
        super().__init__(parent)

        self.setMouseTracking(True)
        self.setAutoFillBackground(True)
        self.star_rating = StarRating()

    def sizeHint(self):
        """ Tell the caller how big we are. """
        return self.star_rating.sizeHint()

    def paintEvent(self, event):
        """ Paint the editor, offloading the work to the StarRating class. """
        with QPainter(self) as painter:
            self.star_rating.paint(painter, self.rect(), self.palette(), isEditable=True)

    def mouseMoveEvent(self, event):
        """ As the mouse moves inside the editor, track the position and
            update the editor to display as many stars as necessary.
        """
        star = self.star_at_position(event.x())

        if (star != self.star_rating.star_count) and (star != -1):
            self.star_rating.star_count = star
            self.update()

    def mouseReleaseEvent(self, event):
        """ Once the user has clicked his/her chosen star rating, tell the
            delegate we're done editing.
        """
        self.editing_finished.emit()

    def star_at_position(self, x):
        """ Calculate which star the user's mouse cursor is currently
            hovering over.
        """
        star = (x / (self.star_rating.sizeHint().width() / self.star_rating.MAX_STAR_COUNT)) + 1
        if (star <= 0) or (star > self.star_rating.MAX_STAR_COUNT):
            return -1

        return star



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for __hash__'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QTreeWidgetItem
from helper.usesqapplication import UsesQApplication


class HashableTest(UsesQApplication):

    def testQTreeWidgetItemHash(self):
        h = {}
        obj = QTreeWidgetItem()
        h[obj] = 2
        self.assertTrue(h.get(obj), 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QVersionNumber'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QVersionNumber


class QVersionNumberTest(unittest.TestCase):
    def testFromString(self):
        versionString = '5.9.2'
        version = QVersionNumber.fromString(versionString)
        self.assertTrue(not version.isNull())
        self.assertEqual(version.majorVersion(), 5)
        self.assertEqual(version.minorVersion(), 9)
        self.assertEqual(version.microVersion(), 2)
        self.assertEqual(version.toString(), versionString)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for Point class'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Point


class PointTest(unittest.TestCase):
    '''Test case for Point class, including operator overloads.'''

    def assertRaises(self, *args, **kwds):
        if not hasattr(sys, "pypy_version_info"):
            # PYSIDE-535: PyPy complains "Fatal RPython error: NotImplementedError"
            return super().assertRaises(*args, **kwds)

    def testConstructor(self):
        '''Test Point class constructor.'''
        pt = Point(5.0, 2.3)
        self.assertEqual(pt.x(), 5.0)
        self.assertEqual(pt.y(), 2.3)

    def testPlusOperator(self):
        '''Test Point class + operator.'''
        pt1 = Point(5.0, 2.3)
        pt2 = Point(0.5, 3.2)
        self.assertEqual(pt1 + pt2, Point(5.0 + 0.5, 2.3 + 3.2))

    def testEqualOperator(self):
        '''Test Point class == operator.'''
        pt1 = Point(5.0, 2.3)
        pt2 = Point(5.0, 2.3)
        pt3 = Point(0.5, 3.2)
        self.assertTrue(pt1 == pt1)
        self.assertTrue(pt1 == pt2)
        self.assertFalse(pt1 == pt3)
        self.assertFalse(pt1 == object())

    def testNotEqualOperator(self):
        '''Test Point class != operator.'''
        pt1 = Point(5.0, 2.3)
        pt2 = Point(5.0, 2.3)
        # This test no longer makes sense because we always supply default `==`, `!=`.
        #self.assertRaises(NotImplementedError, pt1.__ne__, pt2)
        # Since we use the default identity comparison, this results in `!=` .
        self.assertTrue(pt1 != pt2)

    def testReturnNewCopy(self):
        '''Point returns a copy of itself.'''
        pt1 = Point(1.1, 2.3)
        pt2 = pt1.copy()
        self.assertEqual(pt1, pt2)
        pt2 += pt1
        self.assertFalse(pt1 == pt2)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testReturnConstPointer(self):
        '''Point returns a const pointer for itself.'''
        pt1 = Point(5.0, 2.3)
        refcount1 = sys.getrefcount(pt1)
        pt2 = pt1.getSelf()
        self.assertEqual(pt1, pt2)
        self.assertEqual(sys.getrefcount(pt1), refcount1 + 1)
        self.assertEqual(sys.getrefcount(pt1), sys.getrefcount(pt2))

    def testUintOverflow(self):
        pt1 = Point(0.0, 0.0)
        self.assertRaises(OverflowError, pt1.setXAsUint, 840835495615213080)
        self.assertEqual(pt1.x(), 0.0)

    def testAddedOperator(self):
        p = Point(0.0, 0.0)
        r = p - 'Hi'
        self.assertEqual(r, 'Hi')

        # now the reverse op.
        r = 'Hi' - p
        self.assertEqual(r, 'Hi')

    def testModifiedMethod(self):
        pt1 = Point(0.0, 0.0)
        pt2 = Point(10.0, 10.0)
        expected = Point((pt1.x() + pt2.x()) / 2.0, (pt1.y() + pt2.y()) / 2.0)
        self.assertEqual(pt1.midpoint(pt2), expected)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import Property, QSize
from PySide6.QtWidgets import QApplication, QMainWindow, QWidget

is_pypy = hasattr(sys, "pypy_version_info")
if not is_pypy:
    from PySide6.support import feature  # noqa

"""
snake_prop_feature_test.py
--------------------------

Test the snake_case and true_property feature.

This works now, including class properties.
"""


class Window(QWidget):
    def __init__(self):
        super().__init__()


@unittest.skipIf(is_pypy, "__feature__ cannot yet be used with PyPy")
class FeatureTest(unittest.TestCase):
    def setUp(self):
        qApp or QApplication()  # noqa
        feature.reset()

    def tearDown(self):
        feature.reset()
        qApp.shutdown()  # noqa

    def testRenamedFunctions(self):
        window = Window()
        window.setWindowTitle('camelCase')

        # and now the same with snake_case enabled
        from __feature__ import snake_case  # noqa

        # Works with the same window! window = Window()
        window.set_window_title('snake_case')

    def testPropertyAppearVanish(self):
        window = Window()

        self.assertTrue(callable(window.isModal))
        with self.assertRaises(AttributeError):
            window.modal

        from __feature__ import snake_case, true_property  # noqa
        # PYSIDE-1548: Make sure that another import does not clear the features.
        import sys  # noqa

        self.assertTrue(isinstance(QWidget.modal, property))
        self.assertTrue(isinstance(window.modal, bool))
        with self.assertRaises(AttributeError):
            window.isModal

        # switching back
        feature.reset()

        self.assertTrue(callable(window.isModal))
        with self.assertRaises(AttributeError):
            window.modal

    def testClassProperty(self):
        from __feature__ import snake_case, true_property  # noqa
        # We check the class...
        self.assertEqual(type(QApplication.quit_on_last_window_closed), bool)
        x = QApplication.quit_on_last_window_closed
        QApplication.quit_on_last_window_closed = not x
        self.assertEqual(QApplication.quit_on_last_window_closed, not x)
        # ... and now the instance.
        self.assertEqual(type(qApp.quit_on_last_window_closed), bool)  # noqa
        x = qApp.quit_on_last_window_closed  # noqa
        qApp.quit_on_last_window_closed = not x  # noqa
        self.assertEqual(qApp.quit_on_last_window_closed, not x)  # noqa
        # make sure values are equal
        self.assertEqual(qApp.quit_on_last_window_closed,  # noqa
                         QApplication.quit_on_last_window_closed)

    def testUserClassNotAffected(self):
        FunctionType = type(lambda: 42)
        # Note: the types module does not have MethodDescriptorType in low versions.
        MethodDescriptorType = type(str.split)

        class UserClass(QWidget):

            def someFunc1(self):
                pass

            @staticmethod
            def someFunc2(a, b):
                pass

        inspect = UserClass.__dict__
        self.assertTrue(isinstance(inspect["someFunc1"], FunctionType))
        self.assertTrue(isinstance(inspect["someFunc2"], staticmethod))
        self.assertTrue(isinstance(UserClass.someFunc2, FunctionType))
        self.assertTrue(isinstance(UserClass.addAction, MethodDescriptorType))

        from __feature__ import snake_case  # noqa

        inspect = UserClass.__dict__
        self.assertTrue(isinstance(inspect["someFunc1"], FunctionType))
        self.assertTrue(isinstance(inspect["someFunc2"], staticmethod))
        self.assertTrue(isinstance(UserClass.someFunc2, FunctionType))
        self.assertTrue(isinstance(UserClass.add_action, MethodDescriptorType))

    def testTrueProperyCanOverride(self):
        from __feature__ import true_property  # noqa

        class CustomWidget(QWidget):
            global prop_result
            prop_result = None

            @Property(QSize)
            def minimumSizeHint(self):
                global prop_result
                print("called")
                prop_result = super().minimumSizeHint
                return prop_result

        window = QMainWindow()
        window.setCentralWidget(CustomWidget(window))
        window.show()
        self.assertTrue(isinstance(prop_result, QSize))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QRect'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QPoint, QRect, QRectF


class RectConstructor(unittest.TestCase):

    def testDefault(self):
        # QRect()
        obj = QRect()

        self.assertTrue(obj.isNull())

    def testConstructorQPoint(self):
        topLeft = QPoint(3, 0)
        bottomRight = QPoint(0, 3)

        rect1 = QRect(topLeft, bottomRight)
        rect2 = QRect(topLeft, bottomRight)

        self.assertEqual(rect1, rect2)


class RectOperator(unittest.TestCase):
    '''Test case for QRect operators'''

    def testEqual(self):
        '''QRect == QRect
        Note: operator == must be working as it's the main check
        for correctness'''
        rect1 = QRect()
        rect2 = QRect()
        self.assertEqual(rect1, rect2)

        rect1 = QRect(0, 4, 100, 300)
        rect2 = QRect(0, 4, 100, 300)
        self.assertEqual(rect1, rect2)

    def testNullRectIntersection(self):
        # QRect & QRect for null rects
        rect1 = QRect()
        rect2 = QRect()
        rect3 = rect1 & rect2
        self.assertEqual(rect3, rect1)
        self.assertEqual(rect3, rect2)

    def testNoIntersect(self):
        '''QRect & QRect for non-intersecting QRects
        Non-intersecting QRects return a 'null' QRect for operator &'''
        rect1 = QRect(10, 10, 5, 5)
        rect2 = QRect(20, 20, 5, 5)
        rect3 = rect1 & rect2
        self.assertEqual(rect3, QRect())

    def testIntersectPartial(self):
        # QRect & QRect for partial intersections
        rect1 = QRect(10, 10, 10, 10)
        rect2 = QRect(15, 15, 10, 10)
        rect3 = rect1 & rect2
        self.assertEqual(rect3, QRect(15, 15, 5, 5))

    def testIntersetEnclosed(self):
        # QRect & QRect for a qrect inside another
        rect1 = QRect(10, 10, 20, 20)
        rect2 = QRect(15, 15, 5, 5)
        rect3 = rect1 & rect2
        self.assertEqual(rect3, rect2)

    def testNullRectIntersectBounding(self):
        # QRect | QRect for null rects
        rect1 = QRect()
        rect2 = QRect()
        rect3 = rect1 & rect2
        self.assertEqual(rect3, rect1)
        self.assertEqual(rect3, rect2)

    def testNoIntersectBounding(self):
        '''QRect | QRect for non-intersecting QRects
        Non-intersecting QRects return a greater QRect for operator |'''
        rect1 = QRect(10, 10, 5, 5)
        rect2 = QRect(20, 20, 5, 5)
        rect3 = rect1 | rect2
        self.assertEqual(rect3, QRect(10, 10, 15, 15))

    def testBoundingPartialIntersection(self):
        # QRect | QRect for partial intersections
        rect1 = QRect(10, 10, 10, 10)
        rect2 = QRect(15, 15, 10, 10)
        rect3 = rect1 | rect2
        self.assertEqual(rect3, QRect(10, 10, 15, 15))

    def testBoundingEnclosed(self):
        # QRect | QRect for a qrect inside another
        rect1 = QRect(10, 10, 20, 20)
        rect2 = QRect(15, 15, 5, 5)
        rect3 = rect1 | rect2
        self.assertEqual(rect3, rect1)

    def testGetCoordsAndRect(self):
        rect1 = QRect(1, 2, 3, 4)
        self.assertEqual(rect1.getRect(), (1, 2, 3, 4))
        self.assertEqual(rect1.getCoords(), (1, 2, 3, 5))

        rect1 = QRectF(1, 2, 3, 4)
        self.assertEqual(rect1.getRect(), (1, 2, 3, 4))
        self.assertEqual(rect1.getCoords(), (1, 2, 4, 6))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""
signature_bootstrap.py
----------------------

This file was originally directly embedded into the C source.
After it grew more and more, I now prefer to have it as Python file.

Meanwhile, there is also no more a stub loader necessary:
Because we meanwhile have embedding support, we could also load this file
directly from a .pyc file.

This file replaces the hard to read Python stub in 'signature.cpp', and we
could distinguish better between bootstrap related functions and loader
functions.
It is embedded into 'signature.cpp' as "embed/signature_bootstrap.inc".

# PYSIDE-1436: Python 3.10 had a problem with EmbeddableZipImporter because the
imports were in the functions. Moved them outside into the globals.
"""

import base64
import io
import os
import sys
import traceback
import zipfile

from contextlib import contextmanager
from importlib.machinery import ModuleSpec
from pathlib import Path


recursion_trap = 0


def bootstrap():

    global recursion_trap
    if recursion_trap:
        # we are probably called from outside, already
        print("Recursion occurred in Bootstrap. Did you start by hand? Then it's ok.")
        print("But you should trigger start by '_init_pyside_extension()', only!")
    recursion_trap += 1

    @contextmanager
    def ensure_shibokensupport(target, support_path):
        # Make sure that we always have the shibokensupport containing package first.
        # Also remove any prior loaded module of this name, just in case.
        # PYSIDE-1621: support_path can also be a finder instance.
        target.insert(0, support_path)

        sbks = "shibokensupport"
        if sbks in sys.modules:
            del sys.modules[sbks]
        prefix = sbks + "."
        for key in list(key for key in sys.modules if key.startswith(prefix)):
            del sys.modules[key]
        try:
            import shibokensupport  # noqa: F401
            yield
        except Exception as e:
            f = sys.stderr
            print("Problem importing shibokensupport:", file=f)
            print(f"{e.__class__.__name__}: {e}", file=f)
            traceback.print_exc()
            print("sys.path:", file=f)
            for p in sys.path:
                print("  " + p, file=f)
            f.flush()
            sys.exit(-1)
        target.remove(support_path)

    # Here we decide if we re-incarnate the embedded files or use embedding.
    incarnated = find_incarnated_files()
    if incarnated:
        target, support_path = sys.path, os.fspath(incarnated)
    else:
        target, support_path = prepare_zipfile()
    with ensure_shibokensupport(target, support_path):
        from shibokensupport.signature import loader
    return loader


def find_incarnated_files():
    """Newer functionality: This function checks if the support directory exist and
       returns it. If does not exist, we try to create it and return it. Otherwise,
       we return None."""
    import shiboken6 as root
    files_dir = Path(root.__file__).resolve().parent / "files.dir"
    handle_embedding_switch(files_dir)
    if files_dir.exists():
        sys.path.insert(0, os.fspath(files_dir))
        # Note: To avoid recursion problems, we need to preload the loader.
        # But that has the side-effect that we need to delay the feature
        # initialization until all function pointers are set.
        # See `post_init_func` in signature_globals.cpp .
        import shibokensupport.signature.loader  # noqa: F401
        del sys.path[0]
        return files_dir
    return None


def handle_embedding_switch(files_dir):
    """
    This handles the optional environment variable `SBK_EMBED`
    if not set             : do nothing
    if set to 0, false, no : de-virtualize the Python files
    if set to 1, true, yes : virtualize again (delete "files.dir")
    """
    env_name = "SBK_EMBED"
    env_var = os.environ.get(env_name)
    if not env_var:
        return
    if env_var.lower() in ("1", "t", "true", "y", "yes"):
        import shutil
        shutil.rmtree(files_dir, ignore_errors=True)
    elif env_var.lower() in ("0", "f", "false", "n", "no"):
        reincarnate_files(files_dir)


def reincarnate_files(files_dir):
    target, zip = prepare_zipfile()
    names = (_ for _ in zip.zfile.namelist() if _.endswith(".py"))
    try:
        # First check mkdir to get an error when we cannot write.
        files_dir.mkdir(exist_ok=True)
    except os.error:
        print(f"SBK_EMBED=False: Warning: Cannot write into {files_dir}")
        return None
    try:
        # Then check for a real error when unpacking the zip file.
        zip.zfile.extractall(path=files_dir, members=names)
        return files_dir
    except Exception as e:
        print(f"{e.__class__.__name__}: {e}", file=sys.stderr)
        traceback.print_exc()
        raise

# New functionality: Loading from a zip archive.
# There exists the zip importer, but as it is written, only real zip files are
# supported. Before I will start an own implementation, it is easiest to use
# a temporary zip file.
# PYSIDE-1621: make zip file access totally virtual


def prepare_zipfile():
    """
    Old approach:

    Write the zip file to a real file and return its name.
    It will be implicitly opened as such when we add the name to sys.path .

    New approach (Python 3, only):

    Use EmbeddableZipImporter and pass the zipfile structure directly.
    The sys.path way does not work, instead we need to use sys.meta_path .
    See https://docs.python.org/3/library/sys.html#sys.meta_path
    """

    # 'zipstring_sequence' comes from signature.cpp
    zipbytes = base64.b64decode(''.join(zipstring_sequence))   # noqa: F821
    vzip = zipfile.ZipFile(io.BytesIO(zipbytes))
    return sys.meta_path, EmbeddableZipImporter(vzip)


class EmbeddableZipImporter:

    def __init__(self, zip_file):
        def p2m(filename):
            if filename.endswith("/__init__.py"):
                return filename[:-12].replace("/", ".")
            if filename.endswith(".py"):
                return filename[:-3].replace("/", ".")
            return None

        self.zfile = zip_file
        self._mod2path = {p2m(_.filename): _.filename for _ in zip_file.filelist}

    def find_spec(self, fullname, path, target=None):
        path = self._mod2path.get(fullname)
        return ModuleSpec(fullname, self) if path else None

    def create_module(self, spec):
        return None

    def exec_module(self, module):
        fullname = module.__spec__.name
        filename = self._mod2path[fullname]
        with self.zfile.open(filename, "r") as f:   # "rb" not for zipfile
            codeob = compile(f.read(), filename, "exec")
            exec(codeob, module.__dict__)
        module.__file__ = filename
        module.__loader__ = self
        if filename.endswith("/__init__.py"):
            module.__path__ = []
            module.__package__ = fullname
        else:
            module.__package__ = fullname.rpartition('.')[0]
        sys.modules[fullname] = module

# eof



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit test for Location'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtLocation import QGeoServiceProvider  # noqa: E402


class QLocationTestCase(unittest.TestCase):
    def test(self):
        geoServiceProvider = QGeoServiceProvider("none")
        self.assertEqual(geoServiceProvider.errorString(),
                         'The geoservices provider none is not supported.')


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Helper classes and functions'''

import gc
import sys
import unittest

# This version avoids explicit import in order to adapt to the
# import decision of the main module.
# This should work with every compatible library.
# Replaces the QtGui and QtCore versions as well.


class UsesQApplication(unittest.TestCase):
    '''Helper class to provide Q(Core|Gui|)Application instances
    Just connect or call self.exit_app_cb. When called, will ask
    self.app to exit.
    '''

    def setUp(self):
        '''Creates the QApplication instance'''
        module = sys.modules[sorted(_ for _ in sys.modules
                                    if _.endswith((".QtCore", ".QtGui", ".QtWidgets")))[-1]]
        found = module.__name__.rsplit(".")[-1]
        cls = getattr(module, {"QtWidgets": "QApplication",
                               "QtGui": "QGuiApplication",
                               "QtCore": "QCoreApplication"}[found])
        # Simple way of making instance a singleton
        super().setUp()
        self.app = cls.instance() or cls([])

    def tearDown(self):
        '''Deletes the reference owned by self'''
        del self.app
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super().tearDown()

    def exit_app_cb(self):
        '''Quits the application'''
        self.app.exit(0)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from enum import Enum
from math import sin, cos, degrees, sqrt

from PySide6.QtCore import QObject, Signal, Slot, Qt, QRandomGenerator
from PySide6.QtGui import QVector2D, QVector3D
from PySide6.QtGraphs import (QAbstract3DSeries,
                              QScatterDataItem, QScatterDataProxy,
                              QScatter3DSeries, QtGraphs3D, QGraphsTheme)


NUMBER_OF_ITEMS = 10000
CURVE_DIVIDER = 7.5
LOWER_NUMBER_OF_ITEMS = 900
LOWER_CURVE_DIVIDER = 0.75


class InputState(Enum):
    StateNormal = 0
    StateDraggingX = 1
    StateDraggingZ = 2
    StateDraggingY = 3


class ScatterDataModifier(QObject):

    backgroundVisibleChanged = Signal(bool)
    gridVisibleChanged = Signal(bool)
    shadowQualityChanged = Signal(int)

    def __init__(self, scatter, parent):
        super().__init__(parent)

        self._graph = scatter

        self._style = QAbstract3DSeries.Mesh.Sphere
        self._smooth = True
        self._autoAdjust = True
        self._itemCount = LOWER_NUMBER_OF_ITEMS
        self._CURVE_DIVIDER = LOWER_CURVE_DIVIDER

        self._graph.setShadowQuality(QtGraphs3D.ShadowQuality.SoftHigh)
        self._graph.setCameraPreset(QtGraphs3D.CameraPreset.Front)
        self._graph.setCameraZoomLevel(80.0)
        self._graph.activeTheme().setTheme(QGraphsTheme.Theme.MixSeries)
        self._graph.activeTheme().setColorScheme(QGraphsTheme.ColorScheme.Dark)

        self._proxy = QScatterDataProxy()
        self._series = QScatter3DSeries(self._proxy)
        self._series.setItemLabelFormat("@xTitle: @xLabel @yTitle: @yLabel @zTitle: @zLabel")
        self._series.setMeshSmooth(self._smooth)
        self._graph.addSeries(self._series)
        self._preset = QtGraphs3D.CameraPreset.FrontLow.value

        self._state = InputState.StateNormal
        self._dragSpeedModifier = float(15)

        self._graph.selectedElementChanged.connect(self.handleElementSelected)
        self._graph.dragged.connect(self.handleAxisDragging)
        self._graph.setDragButton(Qt.MouseButton.LeftButton)

        self.addData()

    def addData(self):
        # Configure the axes according to the data
        self._graph.axisX().setTitle("X")
        self._graph.axisY().setTitle("Y")
        self._graph.axisZ().setTitle("Z")

        dataArray = []
        limit = int(sqrt(self._itemCount) / 2.0)
        for i in range(-limit, limit):
            for j in range(-limit, limit):
                x = float(i) + 0.5
                y = cos(degrees(float(i * j) / self._CURVE_DIVIDER))
                z = float(j) + 0.5
                dataArray.append(QScatterDataItem(QVector3D(x, y, z)))

        self._graph.seriesList()[0].dataProxy().resetArray(dataArray)

    @Slot(int)
    def changeStyle(self, style):
        comboBox = self.sender()
        if comboBox:
            self._style = comboBox.itemData(style)
            if self._graph.seriesList():
                self._graph.seriesList()[0].setMesh(self._style)

    @Slot(int)
    def setSmoothDots(self, smooth):
        self._smooth = smooth == Qt.CheckState.Checked
        series = self._graph.seriesList()[0]
        series.setMeshSmooth(self._smooth)

    @Slot(int)
    def changeTheme(self, theme):
        currentTheme = self._graph.activeTheme()
        currentTheme.setTheme(QGraphsTheme.Theme(theme))
        self.backgroundVisibleChanged.emit(currentTheme.isPlotAreaBackgroundVisible())
        self.gridVisibleChanged.emit(currentTheme.isGridVisible())

    @Slot()
    def changePresetCamera(self):
        self._graph.setCameraPreset(QtGraphs3D.CameraPreset(self._preset))

        self._preset += 1
        if self._preset > QtGraphs3D.CameraPreset.DirectlyBelow.value:
            self._preset = QtGraphs3D.CameraPreset.FrontLow.value

    @Slot(QtGraphs3D.ShadowQuality)
    def shadowQualityUpdatedByVisual(self, sq):
        self.shadowQualityChanged.emit(sq.value)

    @Slot(QtGraphs3D.ElementType)
    def handleElementSelected(self, type):
        if type == QtGraphs3D.ElementType.AxisXLabel:
            self._state = InputState.StateDraggingX
        elif type == QtGraphs3D.ElementType.AxisYLabel:
            self._state = InputState.StateDraggingY
        elif type == QtGraphs3D.ElementType.AxisZLabel:
            self._state = InputState.StateDraggingZ
        else:
            self._state = InputState.StateNormal

    @Slot(QVector2D)
    def handleAxisDragging(self, delta):
        distance = 0.0
        # Get scene orientation from active camera
        xRotation = self._graph.cameraXRotation()
        yRotation = self._graph.cameraYRotation()

        # Calculate directional drag multipliers based on rotation
        xMulX = cos(degrees(xRotation))
        xMulY = sin(degrees(xRotation))
        zMulX = sin(degrees(xRotation))
        zMulY = cos(degrees(xRotation))

        # Get the drag amount
        move = delta.toPoint()

        # Flip the effect of y movement if we're viewing from below
        yMove = -move.y() if yRotation < 0 else move.y()

        # Adjust axes
        if self._state == InputState.StateDraggingX:
            axis = self._graph.axisX()
            distance = (move.x() * xMulX - yMove * xMulY) / self._dragSpeedModifier
            axis.setRange(axis.min() - distance, axis.max() - distance)
        elif self._state == InputState.StateDraggingZ:
            axis = self._graph.axisZ()
            distance = (move.x() * zMulX + yMove * zMulY) / self._dragSpeedModifier
            axis.setRange(axis.min() + distance, axis.max() + distance)
        elif self._state == InputState.StateDraggingY:
            axis = self._graph.axisY()
            # No need to use adjusted y move here
            distance = move.y() / self._dragSpeedModifier
            axis.setRange(axis.min() + distance, axis.max() + distance)

    @Slot(int)
    def changeShadowQuality(self, quality):
        sq = QtGraphs3D.ShadowQuality(quality)
        self._graph.setShadowQuality(sq)

    @Slot(int)
    def setBackgroundVisible(self, state):
        enabled = state == Qt.CheckState.Checked
        self._graph.activeTheme().setPlotAreaBackgroundVisible(enabled)

    @Slot(int)
    def setGridVisible(self, state):
        self._graph.activeTheme().setGridVisible(state == Qt.Checked.value)

    @Slot()
    def toggleItemCount(self):
        if self._itemCount == NUMBER_OF_ITEMS:
            self._itemCount = LOWER_NUMBER_OF_ITEMS
            self._CURVE_DIVIDER = LOWER_CURVE_DIVIDER
        else:
            self._itemCount = NUMBER_OF_ITEMS
            self._CURVE_DIVIDER = CURVE_DIVIDER

        self._graph.seriesList()[0].dataProxy().resetArray([])
        self.addData()

    @Slot()
    def toggleRanges(self):
        if not self._autoAdjust:
            self._graph.axisX().setAutoAdjustRange(True)
            self._graph.axisZ().setAutoAdjustRange(True)
            self._dragSpeedModifier = 1.5
            self._autoAdjust = True
        else:
            self._graph.axisX().setRange(-10.0, 10.0)
            self._graph.axisZ().setRange(-10.0, 10.0)
            self._dragSpeedModifier = float(15)
            self._autoAdjust = False

    def adjust_minimum_range(self, range):
        if self._itemCount == LOWER_NUMBER_OF_ITEMS:
            range *= 1.45
        else:
            range *= 4.95

        self._graph.axisX().setMin(range)
        self._graph.axisZ().setMin(range)
        self._autoAdjust = False

    def adjust_maximum_range(self, range):
        if self._itemCount == LOWER_NUMBER_OF_ITEMS:
            range *= 1.45
        else:
            range *= 4.95

        self._graph.axisX().setMax(range)
        self._graph.axisZ().setMax(range)
        self._autoAdjust = False

    def rand_vector() -> QVector3D:
        generator = QRandomGenerator.global_()
        x = float(generator.bounded(100)) / 2.0 - float(generator.bounded(100)) / 2.0
        y = float(generator.bounded(100)) / 100.0 - float(generator.bounded(100)) / 100.0
        z = float(generator.bounded(100)) / 2.0 - float(generator.bounded(100)) / 2.0
        return QVector3D(x, y, z)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

# Test case for PySide bug 634
# http://bugs.pyside.org/show_bug.cgi?id=634
# Marcus Lindblom <macke@yar.nu>; 2011-02-16

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication


class Mock:
    def __init__(self):
        self.called = False
        self.return_value = None

    def __call__(self, *args, **kwargs):
        self.called = True
        return self.return_value


class MockClassTest(unittest.TestCase):
    def testMockQCoreApplication(self):
        mock = Mock()
        setattr(QCoreApplication, 'instance', mock)
        QCoreApplication.instance()
        self.assertTrue(mock.called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""
PySide6 port of Qt Quick Controls Contact List example from Qt v6.x
"""
import sys
from pathlib import Path
from PySide6.QtGui import QGuiApplication
from PySide6.QtQml import QQmlApplicationEngine

from contactmodel import ContactModel  # noqa: F401

if __name__ == '__main__':
    app = QGuiApplication(sys.argv)
    app.setOrganizationName("QtProject")
    app.setApplicationName("ContactsList")
    engine = QQmlApplicationEngine()

    engine.addImportPath(Path(__file__).parent)
    engine.loadFromModule("Contact", "ContactList")

    if not engine.rootObjects():
        sys.exit(-1)

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""
fix-complaints.py

This module fixes the buildbot messages of external python files.
Run it once after copying a new version. It is idem-potent, unless
you are changing messages (what I did, of course :-) .
"""

import glob
from pathlib import Path

patched_file_patterns = "backport_inspect.py typing27.py python_minilib_*.py"

offending_words = {
    "behavio""ur": "behavior",
    "at""least": "at_least",
    "reali""sed": "realized",
}

utf8_line = "# This Python file uses the following encoding: utf-8\n"
marker_line = f"# It has been edited by {Path(__file__).name} .\n"


def patch_file(fname):
    with fname.open() as f:
        lines = f.readlines()
    dup = lines[:]
    for idx, line in enumerate(lines):
        for word, repl in offending_words.items():
            if word in line:
                lines[idx] = line.replace(word, repl)
                print(f"line:{line!r} {word!r}->{repl!r}")
    if lines[0].strip() != utf8_line.strip():
        lines[:0] = [utf8_line, "\n"]
    if lines[1] != marker_line:
        lines[1:1] = marker_line
    if lines != dup:
        with open(fname, "w") as f:
            f.write("".join(lines))


def doit():
    dirname = Path(__file__).parent
    patched_files = []
    for name in patched_file_patterns.split():
        pattern = dirname / name
        patched_files += glob.glob(pattern)
    for fname in patched_files:
        print("Working on", fname)
        patch_file(fname)


if __name__ == "__main__":
    doit()

# end of file



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import datetime
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QDateTime, QDate, QTime


class TestQDate (unittest.TestCase):
    def testDateConversion(self):
        dateTime = QDateTime(QDate(2011, 5, 17), QTime(11, 1, 14, 15))
        dateTimePy = QDateTime(datetime.date(2011, 5, 17), datetime.time(11, 1, 14, 15000))
        self.assertEqual(dateTime, dateTimePy)

    def testDateTimeConversion(self):
        dateTime = QDateTime(QDate(2011, 5, 17), QTime(11, 1, 14, 15))
        dateTimePy = QDateTime(datetime.datetime(2011, 5, 17, 11, 1, 14, 15000))
        self.assertEqual(dateTime, dateTimePy)

    def testDateTimeNow(self):
        py = datetime.datetime.now()
        qt = QDateTime(py)
        self.assertEqual(qt, py)


if __name__ == '__main__':
    unittest.main()
